# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# AUTO-GENERATED FROM `$REPO_ROOT/templates/src/python/grpcio/grpc_core_dependencies.py.template`!!!

CORE_SOURCE_FILES = [
  'src/core/lib/profiling/basic_timers.c',
  'src/core/lib/profiling/stap_timers.c',
  'src/core/lib/support/alloc.c',
  'src/core/lib/support/avl.c',
  'src/core/lib/support/backoff.c',
  'src/core/lib/support/cmdline.c',
  'src/core/lib/support/cpu_iphone.c',
  'src/core/lib/support/cpu_linux.c',
  'src/core/lib/support/cpu_posix.c',
  'src/core/lib/support/cpu_windows.c',
  'src/core/lib/support/env_linux.c',
  'src/core/lib/support/env_posix.c',
  'src/core/lib/support/env_windows.c',
  'src/core/lib/support/histogram.c',
  'src/core/lib/support/host_port.c',
  'src/core/lib/support/log.c',
  'src/core/lib/support/log_android.c',
  'src/core/lib/support/log_linux.c',
  'src/core/lib/support/log_posix.c',
  'src/core/lib/support/log_windows.c',
  'src/core/lib/support/murmur_hash.c',
  'src/core/lib/support/slice.c',
  'src/core/lib/support/slice_buffer.c',
  'src/core/lib/support/stack_lockfree.c',
  'src/core/lib/support/string.c',
  'src/core/lib/support/string_posix.c',
  'src/core/lib/support/string_util_windows.c',
  'src/core/lib/support/string_windows.c',
  'src/core/lib/support/subprocess_posix.c',
  'src/core/lib/support/subprocess_windows.c',
  'src/core/lib/support/sync.c',
  'src/core/lib/support/sync_posix.c',
  'src/core/lib/support/sync_windows.c',
  'src/core/lib/support/thd.c',
  'src/core/lib/support/thd_posix.c',
  'src/core/lib/support/thd_windows.c',
  'src/core/lib/support/time.c',
  'src/core/lib/support/time_posix.c',
  'src/core/lib/support/time_precise.c',
  'src/core/lib/support/time_windows.c',
  'src/core/lib/support/tls_pthread.c',
  'src/core/lib/support/tmpfile_msys.c',
  'src/core/lib/support/tmpfile_posix.c',
  'src/core/lib/support/tmpfile_windows.c',
  'src/core/lib/support/wrap_memcpy.c',
  'src/core/lib/surface/init.c',
  'src/core/lib/channel/channel_args.c',
  'src/core/lib/channel/channel_stack.c',
  'src/core/lib/channel/channel_stack_builder.c',
  'src/core/lib/channel/compress_filter.c',
  'src/core/lib/channel/connected_channel.c',
  'src/core/lib/channel/http_client_filter.c',
  'src/core/lib/channel/http_server_filter.c',
  'src/core/lib/compression/compression.c',
  'src/core/lib/compression/message_compress.c',
  'src/core/lib/debug/trace.c',
  'src/core/lib/http/format_request.c',
  'src/core/lib/http/httpcli.c',
  'src/core/lib/http/parser.c',
  'src/core/lib/iomgr/closure.c',
  'src/core/lib/iomgr/endpoint.c',
  'src/core/lib/iomgr/endpoint_pair_posix.c',
  'src/core/lib/iomgr/endpoint_pair_windows.c',
  'src/core/lib/iomgr/error.c',
  'src/core/lib/iomgr/ev_epoll_linux.c',
  'src/core/lib/iomgr/ev_poll_and_epoll_posix.c',
  'src/core/lib/iomgr/ev_poll_posix.c',
  'src/core/lib/iomgr/ev_posix.c',
  'src/core/lib/iomgr/exec_ctx.c',
  'src/core/lib/iomgr/executor.c',
  'src/core/lib/iomgr/iocp_windows.c',
  'src/core/lib/iomgr/iomgr.c',
  'src/core/lib/iomgr/iomgr_posix.c',
  'src/core/lib/iomgr/iomgr_windows.c',
  'src/core/lib/iomgr/load_file.c',
  'src/core/lib/iomgr/network_status_tracker.c',
  'src/core/lib/iomgr/polling_entity.c',
  'src/core/lib/iomgr/pollset_set_windows.c',
  'src/core/lib/iomgr/pollset_windows.c',
  'src/core/lib/iomgr/resolve_address_posix.c',
  'src/core/lib/iomgr/resolve_address_windows.c',
  'src/core/lib/iomgr/sockaddr_utils.c',
  'src/core/lib/iomgr/socket_utils_common_posix.c',
  'src/core/lib/iomgr/socket_utils_linux.c',
  'src/core/lib/iomgr/socket_utils_posix.c',
  'src/core/lib/iomgr/socket_windows.c',
  'src/core/lib/iomgr/tcp_client_posix.c',
  'src/core/lib/iomgr/tcp_client_windows.c',
  'src/core/lib/iomgr/tcp_posix.c',
  'src/core/lib/iomgr/tcp_server_posix.c',
  'src/core/lib/iomgr/tcp_server_windows.c',
  'src/core/lib/iomgr/tcp_windows.c',
  'src/core/lib/iomgr/time_averaged_stats.c',
  'src/core/lib/iomgr/timer.c',
  'src/core/lib/iomgr/timer_heap.c',
  'src/core/lib/iomgr/udp_server.c',
  'src/core/lib/iomgr/unix_sockets_posix.c',
  'src/core/lib/iomgr/unix_sockets_posix_noop.c',
  'src/core/lib/iomgr/wakeup_fd_eventfd.c',
  'src/core/lib/iomgr/wakeup_fd_nospecial.c',
  'src/core/lib/iomgr/wakeup_fd_pipe.c',
  'src/core/lib/iomgr/wakeup_fd_posix.c',
  'src/core/lib/iomgr/workqueue_posix.c',
  'src/core/lib/iomgr/workqueue_windows.c',
  'src/core/lib/json/json.c',
  'src/core/lib/json/json_reader.c',
  'src/core/lib/json/json_string.c',
  'src/core/lib/json/json_writer.c',
  'src/core/lib/surface/alarm.c',
  'src/core/lib/surface/api_trace.c',
  'src/core/lib/surface/byte_buffer.c',
  'src/core/lib/surface/byte_buffer_reader.c',
  'src/core/lib/surface/call.c',
  'src/core/lib/surface/call_details.c',
  'src/core/lib/surface/call_log_batch.c',
  'src/core/lib/surface/channel.c',
  'src/core/lib/surface/channel_init.c',
  'src/core/lib/surface/channel_ping.c',
  'src/core/lib/surface/channel_stack_type.c',
  'src/core/lib/surface/completion_queue.c',
  'src/core/lib/surface/event_string.c',
  'src/core/lib/surface/lame_client.c',
  'src/core/lib/surface/metadata_array.c',
  'src/core/lib/surface/server.c',
  'src/core/lib/surface/validate_metadata.c',
  'src/core/lib/surface/version.c',
  'src/core/lib/transport/byte_stream.c',
  'src/core/lib/transport/connectivity_state.c',
  'src/core/lib/transport/metadata.c',
  'src/core/lib/transport/metadata_batch.c',
  'src/core/lib/transport/static_metadata.c',
  'src/core/lib/transport/transport.c',
  'src/core/lib/transport/transport_op_string.c',
  'src/core/ext/transport/chttp2/server/secure/server_secure_chttp2.c',
  'src/core/ext/transport/chttp2/transport/bin_decoder.c',
  'src/core/ext/transport/chttp2/transport/bin_encoder.c',
  'src/core/ext/transport/chttp2/transport/chttp2_plugin.c',
  'src/core/ext/transport/chttp2/transport/chttp2_transport.c',
  'src/core/ext/transport/chttp2/transport/frame_data.c',
  'src/core/ext/transport/chttp2/transport/frame_goaway.c',
  'src/core/ext/transport/chttp2/transport/frame_ping.c',
  'src/core/ext/transport/chttp2/transport/frame_rst_stream.c',
  'src/core/ext/transport/chttp2/transport/frame_settings.c',
  'src/core/ext/transport/chttp2/transport/frame_window_update.c',
  'src/core/ext/transport/chttp2/transport/hpack_encoder.c',
  'src/core/ext/transport/chttp2/transport/hpack_parser.c',
  'src/core/ext/transport/chttp2/transport/hpack_table.c',
  'src/core/ext/transport/chttp2/transport/huffsyms.c',
  'src/core/ext/transport/chttp2/transport/incoming_metadata.c',
  'src/core/ext/transport/chttp2/transport/parsing.c',
  'src/core/ext/transport/chttp2/transport/status_conversion.c',
  'src/core/ext/transport/chttp2/transport/stream_lists.c',
  'src/core/ext/transport/chttp2/transport/stream_map.c',
  'src/core/ext/transport/chttp2/transport/timeout_encoding.c',
  'src/core/ext/transport/chttp2/transport/varint.c',
  'src/core/ext/transport/chttp2/transport/writing.c',
  'src/core/ext/transport/chttp2/alpn/alpn.c',
  'src/core/lib/http/httpcli_security_connector.c',
  'src/core/lib/security/context/security_context.c',
  'src/core/lib/security/credentials/composite/composite_credentials.c',
  'src/core/lib/security/credentials/credentials.c',
  'src/core/lib/security/credentials/credentials_metadata.c',
  'src/core/lib/security/credentials/fake/fake_credentials.c',
  'src/core/lib/security/credentials/google_default/credentials_posix.c',
  'src/core/lib/security/credentials/google_default/credentials_windows.c',
  'src/core/lib/security/credentials/google_default/google_default_credentials.c',
  'src/core/lib/security/credentials/iam/iam_credentials.c',
  'src/core/lib/security/credentials/jwt/json_token.c',
  'src/core/lib/security/credentials/jwt/jwt_credentials.c',
  'src/core/lib/security/credentials/jwt/jwt_verifier.c',
  'src/core/lib/security/credentials/oauth2/oauth2_credentials.c',
  'src/core/lib/security/credentials/plugin/plugin_credentials.c',
  'src/core/lib/security/credentials/ssl/ssl_credentials.c',
  'src/core/lib/security/transport/client_auth_filter.c',
  'src/core/lib/security/transport/handshake.c',
  'src/core/lib/security/transport/secure_endpoint.c',
  'src/core/lib/security/transport/security_connector.c',
  'src/core/lib/security/transport/server_auth_filter.c',
  'src/core/lib/security/transport/tsi_error.c',
  'src/core/lib/security/util/b64.c',
  'src/core/lib/security/util/json_util.c',
  'src/core/lib/surface/init_secure.c',
  'src/core/lib/tsi/fake_transport_security.c',
  'src/core/lib/tsi/ssl_transport_security.c',
  'src/core/lib/tsi/transport_security.c',
  'src/core/ext/transport/chttp2/client/secure/secure_channel_create.c',
  'src/core/ext/client_config/channel_connectivity.c',
  'src/core/ext/client_config/client_channel.c',
  'src/core/ext/client_config/client_channel_factory.c',
  'src/core/ext/client_config/client_config.c',
  'src/core/ext/client_config/client_config_plugin.c',
  'src/core/ext/client_config/connector.c',
  'src/core/ext/client_config/default_initial_connect_string.c',
  'src/core/ext/client_config/initial_connect_string.c',
  'src/core/ext/client_config/lb_policy.c',
  'src/core/ext/client_config/lb_policy_factory.c',
  'src/core/ext/client_config/lb_policy_registry.c',
  'src/core/ext/client_config/parse_address.c',
  'src/core/ext/client_config/resolver.c',
  'src/core/ext/client_config/resolver_factory.c',
  'src/core/ext/client_config/resolver_registry.c',
  'src/core/ext/client_config/subchannel.c',
  'src/core/ext/client_config/subchannel_call_holder.c',
  'src/core/ext/client_config/subchannel_index.c',
  'src/core/ext/client_config/uri_parser.c',
  'src/core/ext/transport/chttp2/server/insecure/server_chttp2.c',
  'src/core/ext/transport/chttp2/server/insecure/server_chttp2_posix.c',
  'src/core/ext/transport/chttp2/client/insecure/channel_create.c',
  'src/core/ext/transport/chttp2/client/insecure/channel_create_posix.c',
  'src/core/ext/lb_policy/grpclb/load_balancer_api.c',
  'src/core/ext/lb_policy/grpclb/proto/grpc/lb/v1/load_balancer.pb.c',
  'third_party/nanopb/pb_common.c',
  'third_party/nanopb/pb_decode.c',
  'third_party/nanopb/pb_encode.c',
  'src/core/ext/lb_policy/pick_first/pick_first.c',
  'src/core/ext/lb_policy/round_robin/round_robin.c',
  'src/core/ext/resolver/dns/native/dns_resolver.c',
  'src/core/ext/resolver/sockaddr/sockaddr_resolver.c',
  'src/core/ext/load_reporting/load_reporting.c',
  'src/core/ext/load_reporting/load_reporting_filter.c',
  'src/core/ext/census/context.c',
  'src/core/ext/census/gen/census.pb.c',
  'src/core/ext/census/grpc_context.c',
  'src/core/ext/census/grpc_filter.c',
  'src/core/ext/census/grpc_plugin.c',
  'src/core/ext/census/initialize.c',
  'src/core/ext/census/mlog.c',
  'src/core/ext/census/operation.c',
  'src/core/ext/census/placeholders.c',
  'src/core/ext/census/tracing.c',
  'src/core/plugin_registry/grpc_plugin_registry.c',
  'src/boringssl/err_data.c',
  'third_party/boringssl/crypto/aes/aes.c',
  'third_party/boringssl/crypto/aes/mode_wrappers.c',
  'third_party/boringssl/crypto/asn1/a_bitstr.c',
  'third_party/boringssl/crypto/asn1/a_bool.c',
  'third_party/boringssl/crypto/asn1/a_bytes.c',
  'third_party/boringssl/crypto/asn1/a_d2i_fp.c',
  'third_party/boringssl/crypto/asn1/a_dup.c',
  'third_party/boringssl/crypto/asn1/a_enum.c',
  'third_party/boringssl/crypto/asn1/a_gentm.c',
  'third_party/boringssl/crypto/asn1/a_i2d_fp.c',
  'third_party/boringssl/crypto/asn1/a_int.c',
  'third_party/boringssl/crypto/asn1/a_mbstr.c',
  'third_party/boringssl/crypto/asn1/a_object.c',
  'third_party/boringssl/crypto/asn1/a_octet.c',
  'third_party/boringssl/crypto/asn1/a_print.c',
  'third_party/boringssl/crypto/asn1/a_strnid.c',
  'third_party/boringssl/crypto/asn1/a_time.c',
  'third_party/boringssl/crypto/asn1/a_type.c',
  'third_party/boringssl/crypto/asn1/a_utctm.c',
  'third_party/boringssl/crypto/asn1/a_utf8.c',
  'third_party/boringssl/crypto/asn1/asn1_lib.c',
  'third_party/boringssl/crypto/asn1/asn1_par.c',
  'third_party/boringssl/crypto/asn1/asn_pack.c',
  'third_party/boringssl/crypto/asn1/bio_asn1.c',
  'third_party/boringssl/crypto/asn1/bio_ndef.c',
  'third_party/boringssl/crypto/asn1/f_enum.c',
  'third_party/boringssl/crypto/asn1/f_int.c',
  'third_party/boringssl/crypto/asn1/f_string.c',
  'third_party/boringssl/crypto/asn1/t_bitst.c',
  'third_party/boringssl/crypto/asn1/t_pkey.c',
  'third_party/boringssl/crypto/asn1/tasn_dec.c',
  'third_party/boringssl/crypto/asn1/tasn_enc.c',
  'third_party/boringssl/crypto/asn1/tasn_fre.c',
  'third_party/boringssl/crypto/asn1/tasn_new.c',
  'third_party/boringssl/crypto/asn1/tasn_prn.c',
  'third_party/boringssl/crypto/asn1/tasn_typ.c',
  'third_party/boringssl/crypto/asn1/tasn_utl.c',
  'third_party/boringssl/crypto/asn1/x_bignum.c',
  'third_party/boringssl/crypto/asn1/x_long.c',
  'third_party/boringssl/crypto/base64/base64.c',
  'third_party/boringssl/crypto/bio/bio.c',
  'third_party/boringssl/crypto/bio/bio_mem.c',
  'third_party/boringssl/crypto/bio/buffer.c',
  'third_party/boringssl/crypto/bio/connect.c',
  'third_party/boringssl/crypto/bio/fd.c',
  'third_party/boringssl/crypto/bio/file.c',
  'third_party/boringssl/crypto/bio/hexdump.c',
  'third_party/boringssl/crypto/bio/pair.c',
  'third_party/boringssl/crypto/bio/printf.c',
  'third_party/boringssl/crypto/bio/socket.c',
  'third_party/boringssl/crypto/bio/socket_helper.c',
  'third_party/boringssl/crypto/bn/add.c',
  'third_party/boringssl/crypto/bn/asm/x86_64-gcc.c',
  'third_party/boringssl/crypto/bn/bn.c',
  'third_party/boringssl/crypto/bn/bn_asn1.c',
  'third_party/boringssl/crypto/bn/cmp.c',
  'third_party/boringssl/crypto/bn/convert.c',
  'third_party/boringssl/crypto/bn/ctx.c',
  'third_party/boringssl/crypto/bn/div.c',
  'third_party/boringssl/crypto/bn/exponentiation.c',
  'third_party/boringssl/crypto/bn/gcd.c',
  'third_party/boringssl/crypto/bn/generic.c',
  'third_party/boringssl/crypto/bn/kronecker.c',
  'third_party/boringssl/crypto/bn/montgomery.c',
  'third_party/boringssl/crypto/bn/mul.c',
  'third_party/boringssl/crypto/bn/prime.c',
  'third_party/boringssl/crypto/bn/random.c',
  'third_party/boringssl/crypto/bn/rsaz_exp.c',
  'third_party/boringssl/crypto/bn/shift.c',
  'third_party/boringssl/crypto/bn/sqrt.c',
  'third_party/boringssl/crypto/buf/buf.c',
  'third_party/boringssl/crypto/bytestring/asn1_compat.c',
  'third_party/boringssl/crypto/bytestring/ber.c',
  'third_party/boringssl/crypto/bytestring/cbb.c',
  'third_party/boringssl/crypto/bytestring/cbs.c',
  'third_party/boringssl/crypto/chacha/chacha_generic.c',
  'third_party/boringssl/crypto/chacha/chacha_vec.c',
  'third_party/boringssl/crypto/cipher/aead.c',
  'third_party/boringssl/crypto/cipher/cipher.c',
  'third_party/boringssl/crypto/cipher/derive_key.c',
  'third_party/boringssl/crypto/cipher/e_aes.c',
  'third_party/boringssl/crypto/cipher/e_chacha20poly1305.c',
  'third_party/boringssl/crypto/cipher/e_des.c',
  'third_party/boringssl/crypto/cipher/e_null.c',
  'third_party/boringssl/crypto/cipher/e_rc2.c',
  'third_party/boringssl/crypto/cipher/e_rc4.c',
  'third_party/boringssl/crypto/cipher/e_ssl3.c',
  'third_party/boringssl/crypto/cipher/e_tls.c',
  'third_party/boringssl/crypto/cipher/tls_cbc.c',
  'third_party/boringssl/crypto/cmac/cmac.c',
  'third_party/boringssl/crypto/conf/conf.c',
  'third_party/boringssl/crypto/cpu-arm.c',
  'third_party/boringssl/crypto/cpu-intel.c',
  'third_party/boringssl/crypto/crypto.c',
  'third_party/boringssl/crypto/curve25519/curve25519.c',
  'third_party/boringssl/crypto/curve25519/x25519-x86_64.c',
  'third_party/boringssl/crypto/des/des.c',
  'third_party/boringssl/crypto/dh/check.c',
  'third_party/boringssl/crypto/dh/dh.c',
  'third_party/boringssl/crypto/dh/dh_asn1.c',
  'third_party/boringssl/crypto/dh/params.c',
  'third_party/boringssl/crypto/digest/digest.c',
  'third_party/boringssl/crypto/digest/digests.c',
  'third_party/boringssl/crypto/directory_posix.c',
  'third_party/boringssl/crypto/directory_win.c',
  'third_party/boringssl/crypto/dsa/dsa.c',
  'third_party/boringssl/crypto/dsa/dsa_asn1.c',
  'third_party/boringssl/crypto/ec/ec.c',
  'third_party/boringssl/crypto/ec/ec_asn1.c',
  'third_party/boringssl/crypto/ec/ec_key.c',
  'third_party/boringssl/crypto/ec/ec_montgomery.c',
  'third_party/boringssl/crypto/ec/oct.c',
  'third_party/boringssl/crypto/ec/p224-64.c',
  'third_party/boringssl/crypto/ec/p256-64.c',
  'third_party/boringssl/crypto/ec/p256-x86_64.c',
  'third_party/boringssl/crypto/ec/simple.c',
  'third_party/boringssl/crypto/ec/util-64.c',
  'third_party/boringssl/crypto/ec/wnaf.c',
  'third_party/boringssl/crypto/ecdh/ecdh.c',
  'third_party/boringssl/crypto/ecdsa/ecdsa.c',
  'third_party/boringssl/crypto/ecdsa/ecdsa_asn1.c',
  'third_party/boringssl/crypto/engine/engine.c',
  'third_party/boringssl/crypto/err/err.c',
  'third_party/boringssl/crypto/evp/algorithm.c',
  'third_party/boringssl/crypto/evp/digestsign.c',
  'third_party/boringssl/crypto/evp/evp.c',
  'third_party/boringssl/crypto/evp/evp_asn1.c',
  'third_party/boringssl/crypto/evp/evp_ctx.c',
  'third_party/boringssl/crypto/evp/p_dsa_asn1.c',
  'third_party/boringssl/crypto/evp/p_ec.c',
  'third_party/boringssl/crypto/evp/p_ec_asn1.c',
  'third_party/boringssl/crypto/evp/p_rsa.c',
  'third_party/boringssl/crypto/evp/p_rsa_asn1.c',
  'third_party/boringssl/crypto/evp/pbkdf.c',
  'third_party/boringssl/crypto/evp/sign.c',
  'third_party/boringssl/crypto/ex_data.c',
  'third_party/boringssl/crypto/hkdf/hkdf.c',
  'third_party/boringssl/crypto/hmac/hmac.c',
  'third_party/boringssl/crypto/lhash/lhash.c',
  'third_party/boringssl/crypto/md4/md4.c',
  'third_party/boringssl/crypto/md5/md5.c',
  'third_party/boringssl/crypto/mem.c',
  'third_party/boringssl/crypto/modes/cbc.c',
  'third_party/boringssl/crypto/modes/cfb.c',
  'third_party/boringssl/crypto/modes/ctr.c',
  'third_party/boringssl/crypto/modes/gcm.c',
  'third_party/boringssl/crypto/modes/ofb.c',
  'third_party/boringssl/crypto/obj/obj.c',
  'third_party/boringssl/crypto/obj/obj_xref.c',
  'third_party/boringssl/crypto/pem/pem_all.c',
  'third_party/boringssl/crypto/pem/pem_info.c',
  'third_party/boringssl/crypto/pem/pem_lib.c',
  'third_party/boringssl/crypto/pem/pem_oth.c',
  'third_party/boringssl/crypto/pem/pem_pk8.c',
  'third_party/boringssl/crypto/pem/pem_pkey.c',
  'third_party/boringssl/crypto/pem/pem_x509.c',
  'third_party/boringssl/crypto/pem/pem_xaux.c',
  'third_party/boringssl/crypto/pkcs8/p5_pbe.c',
  'third_party/boringssl/crypto/pkcs8/p5_pbev2.c',
  'third_party/boringssl/crypto/pkcs8/p8_pkey.c',
  'third_party/boringssl/crypto/pkcs8/pkcs8.c',
  'third_party/boringssl/crypto/poly1305/poly1305.c',
  'third_party/boringssl/crypto/poly1305/poly1305_arm.c',
  'third_party/boringssl/crypto/poly1305/poly1305_vec.c',
  'third_party/boringssl/crypto/rand/rand.c',
  'third_party/boringssl/crypto/rand/urandom.c',
  'third_party/boringssl/crypto/rand/windows.c',
  'third_party/boringssl/crypto/rc4/rc4.c',
  'third_party/boringssl/crypto/refcount_c11.c',
  'third_party/boringssl/crypto/refcount_lock.c',
  'third_party/boringssl/crypto/rsa/blinding.c',
  'third_party/boringssl/crypto/rsa/padding.c',
  'third_party/boringssl/crypto/rsa/rsa.c',
  'third_party/boringssl/crypto/rsa/rsa_asn1.c',
  'third_party/boringssl/crypto/rsa/rsa_impl.c',
  'third_party/boringssl/crypto/sha/sha1.c',
  'third_party/boringssl/crypto/sha/sha256.c',
  'third_party/boringssl/crypto/sha/sha512.c',
  'third_party/boringssl/crypto/stack/stack.c',
  'third_party/boringssl/crypto/thread.c',
  'third_party/boringssl/crypto/thread_none.c',
  'third_party/boringssl/crypto/thread_pthread.c',
  'third_party/boringssl/crypto/thread_win.c',
  'third_party/boringssl/crypto/time_support.c',
  'third_party/boringssl/crypto/x509/a_digest.c',
  'third_party/boringssl/crypto/x509/a_sign.c',
  'third_party/boringssl/crypto/x509/a_strex.c',
  'third_party/boringssl/crypto/x509/a_verify.c',
  'third_party/boringssl/crypto/x509/asn1_gen.c',
  'third_party/boringssl/crypto/x509/by_dir.c',
  'third_party/boringssl/crypto/x509/by_file.c',
  'third_party/boringssl/crypto/x509/i2d_pr.c',
  'third_party/boringssl/crypto/x509/pkcs7.c',
  'third_party/boringssl/crypto/x509/t_crl.c',
  'third_party/boringssl/crypto/x509/t_req.c',
  'third_party/boringssl/crypto/x509/t_x509.c',
  'third_party/boringssl/crypto/x509/t_x509a.c',
  'third_party/boringssl/crypto/x509/x509.c',
  'third_party/boringssl/crypto/x509/x509_att.c',
  'third_party/boringssl/crypto/x509/x509_cmp.c',
  'third_party/boringssl/crypto/x509/x509_d2.c',
  'third_party/boringssl/crypto/x509/x509_def.c',
  'third_party/boringssl/crypto/x509/x509_ext.c',
  'third_party/boringssl/crypto/x509/x509_lu.c',
  'third_party/boringssl/crypto/x509/x509_obj.c',
  'third_party/boringssl/crypto/x509/x509_r2x.c',
  'third_party/boringssl/crypto/x509/x509_req.c',
  'third_party/boringssl/crypto/x509/x509_set.c',
  'third_party/boringssl/crypto/x509/x509_trs.c',
  'third_party/boringssl/crypto/x509/x509_txt.c',
  'third_party/boringssl/crypto/x509/x509_v3.c',
  'third_party/boringssl/crypto/x509/x509_vfy.c',
  'third_party/boringssl/crypto/x509/x509_vpm.c',
  'third_party/boringssl/crypto/x509/x509cset.c',
  'third_party/boringssl/crypto/x509/x509name.c',
  'third_party/boringssl/crypto/x509/x509rset.c',
  'third_party/boringssl/crypto/x509/x509spki.c',
  'third_party/boringssl/crypto/x509/x509type.c',
  'third_party/boringssl/crypto/x509/x_algor.c',
  'third_party/boringssl/crypto/x509/x_all.c',
  'third_party/boringssl/crypto/x509/x_attrib.c',
  'third_party/boringssl/crypto/x509/x_crl.c',
  'third_party/boringssl/crypto/x509/x_exten.c',
  'third_party/boringssl/crypto/x509/x_info.c',
  'third_party/boringssl/crypto/x509/x_name.c',
  'third_party/boringssl/crypto/x509/x_pkey.c',
  'third_party/boringssl/crypto/x509/x_pubkey.c',
  'third_party/boringssl/crypto/x509/x_req.c',
  'third_party/boringssl/crypto/x509/x_sig.c',
  'third_party/boringssl/crypto/x509/x_spki.c',
  'third_party/boringssl/crypto/x509/x_val.c',
  'third_party/boringssl/crypto/x509/x_x509.c',
  'third_party/boringssl/crypto/x509/x_x509a.c',
  'third_party/boringssl/crypto/x509v3/pcy_cache.c',
  'third_party/boringssl/crypto/x509v3/pcy_data.c',
  'third_party/boringssl/crypto/x509v3/pcy_lib.c',
  'third_party/boringssl/crypto/x509v3/pcy_map.c',
  'third_party/boringssl/crypto/x509v3/pcy_node.c',
  'third_party/boringssl/crypto/x509v3/pcy_tree.c',
  'third_party/boringssl/crypto/x509v3/v3_akey.c',
  'third_party/boringssl/crypto/x509v3/v3_akeya.c',
  'third_party/boringssl/crypto/x509v3/v3_alt.c',
  'third_party/boringssl/crypto/x509v3/v3_bcons.c',
  'third_party/boringssl/crypto/x509v3/v3_bitst.c',
  'third_party/boringssl/crypto/x509v3/v3_conf.c',
  'third_party/boringssl/crypto/x509v3/v3_cpols.c',
  'third_party/boringssl/crypto/x509v3/v3_crld.c',
  'third_party/boringssl/crypto/x509v3/v3_enum.c',
  'third_party/boringssl/crypto/x509v3/v3_extku.c',
  'third_party/boringssl/crypto/x509v3/v3_genn.c',
  'third_party/boringssl/crypto/x509v3/v3_ia5.c',
  'third_party/boringssl/crypto/x509v3/v3_info.c',
  'third_party/boringssl/crypto/x509v3/v3_int.c',
  'third_party/boringssl/crypto/x509v3/v3_lib.c',
  'third_party/boringssl/crypto/x509v3/v3_ncons.c',
  'third_party/boringssl/crypto/x509v3/v3_pci.c',
  'third_party/boringssl/crypto/x509v3/v3_pcia.c',
  'third_party/boringssl/crypto/x509v3/v3_pcons.c',
  'third_party/boringssl/crypto/x509v3/v3_pku.c',
  'third_party/boringssl/crypto/x509v3/v3_pmaps.c',
  'third_party/boringssl/crypto/x509v3/v3_prn.c',
  'third_party/boringssl/crypto/x509v3/v3_purp.c',
  'third_party/boringssl/crypto/x509v3/v3_skey.c',
  'third_party/boringssl/crypto/x509v3/v3_sxnet.c',
  'third_party/boringssl/crypto/x509v3/v3_utl.c',
  'third_party/boringssl/ssl/custom_extensions.c',
  'third_party/boringssl/ssl/d1_both.c',
  'third_party/boringssl/ssl/d1_clnt.c',
  'third_party/boringssl/ssl/d1_lib.c',
  'third_party/boringssl/ssl/d1_meth.c',
  'third_party/boringssl/ssl/d1_pkt.c',
  'third_party/boringssl/ssl/d1_srtp.c',
  'third_party/boringssl/ssl/d1_srvr.c',
  'third_party/boringssl/ssl/dtls_record.c',
  'third_party/boringssl/ssl/pqueue/pqueue.c',
  'third_party/boringssl/ssl/s3_both.c',
  'third_party/boringssl/ssl/s3_clnt.c',
  'third_party/boringssl/ssl/s3_enc.c',
  'third_party/boringssl/ssl/s3_lib.c',
  'third_party/boringssl/ssl/s3_meth.c',
  'third_party/boringssl/ssl/s3_pkt.c',
  'third_party/boringssl/ssl/s3_srvr.c',
  'third_party/boringssl/ssl/ssl_aead_ctx.c',
  'third_party/boringssl/ssl/ssl_asn1.c',
  'third_party/boringssl/ssl/ssl_buffer.c',
  'third_party/boringssl/ssl/ssl_cert.c',
  'third_party/boringssl/ssl/ssl_cipher.c',
  'third_party/boringssl/ssl/ssl_ecdh.c',
  'third_party/boringssl/ssl/ssl_file.c',
  'third_party/boringssl/ssl/ssl_lib.c',
  'third_party/boringssl/ssl/ssl_rsa.c',
  'third_party/boringssl/ssl/ssl_session.c',
  'third_party/boringssl/ssl/ssl_stat.c',
  'third_party/boringssl/ssl/t1_enc.c',
  'third_party/boringssl/ssl/t1_lib.c',
  'third_party/boringssl/ssl/tls_record.c',
  'third_party/zlib/adler32.c',
  'third_party/zlib/compress.c',
  'third_party/zlib/crc32.c',
  'third_party/zlib/deflate.c',
  'third_party/zlib/gzclose.c',
  'third_party/zlib/gzlib.c',
  'third_party/zlib/gzread.c',
  'third_party/zlib/gzwrite.c',
  'third_party/zlib/infback.c',
  'third_party/zlib/inffast.c',
  'third_party/zlib/inflate.c',
  'third_party/zlib/inftrees.c',
  'third_party/zlib/trees.c',
  'third_party/zlib/uncompr.c',
  'third_party/zlib/zutil.c',
]



# Copyright 2016, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Translates gRPC's client-side API into gRPC's client-side Beta API."""

import grpc
from grpc import _common
from grpc._cython import cygrpc
from grpc.beta import interfaces
from grpc.framework.common import cardinality
from grpc.framework.foundation import future
from grpc.framework.interfaces.face import face

_STATUS_CODE_TO_ABORTION_KIND_AND_ABORTION_ERROR_CLASS = {
    grpc.StatusCode.CANCELLED: (
        face.Abortion.Kind.CANCELLED, face.CancellationError),
    grpc.StatusCode.UNKNOWN: (
        face.Abortion.Kind.REMOTE_FAILURE, face.RemoteError),
    grpc.StatusCode.DEADLINE_EXCEEDED: (
        face.Abortion.Kind.EXPIRED, face.ExpirationError),
    grpc.StatusCode.UNIMPLEMENTED: (
        face.Abortion.Kind.LOCAL_FAILURE, face.LocalError),
}


def _effective_metadata(metadata, metadata_transformer):
  non_none_metadata = () if metadata is None else metadata
  if metadata_transformer is None:
    return non_none_metadata
  else:
    return metadata_transformer(non_none_metadata)


def _credentials(grpc_call_options):
  return None if grpc_call_options is None else grpc_call_options.credentials


def _abortion(rpc_error_call):
  code = rpc_error_call.code()
  pair = _STATUS_CODE_TO_ABORTION_KIND_AND_ABORTION_ERROR_CLASS.get(code)
  error_kind = face.Abortion.Kind.LOCAL_FAILURE if pair is None else pair[0]
  return face.Abortion(
      error_kind, rpc_error_call.initial_metadata(),
      rpc_error_call.trailing_metadata(), code, rpc_error_call.details())


def _abortion_error(rpc_error_call):
  code = rpc_error_call.code()
  pair = _STATUS_CODE_TO_ABORTION_KIND_AND_ABORTION_ERROR_CLASS.get(code)
  exception_class = face.AbortionError if pair is None else pair[1]
  return exception_class(
      rpc_error_call.initial_metadata(), rpc_error_call.trailing_metadata(),
      code, rpc_error_call.details())


class _InvocationProtocolContext(interfaces.GRPCInvocationContext):

  def disable_next_request_compression(self):
    pass  # TODO(https://github.com/grpc/grpc/issues/4078): design, implement.


class _Rendezvous(future.Future, face.Call):

  def __init__(self, response_future, response_iterator, call):
    self._future = response_future
    self._iterator = response_iterator
    self._call = call

  def cancel(self):
    return self._call.cancel()

  def cancelled(self):
    return self._future.cancelled()

  def running(self):
    return self._future.running()

  def done(self):
    return self._future.done()

  def result(self, timeout=None):
    try:
      return self._future.result(timeout=timeout)
    except grpc.RpcError as rpc_error_call:
      raise _abortion_error(rpc_error_call)
    except grpc.FutureTimeoutError:
      raise future.TimeoutError()
    except grpc.FutureCancelledError:
      raise future.CancelledError()

  def exception(self, timeout=None):
    try:
      rpc_error_call = self._future.exception(timeout=timeout)
      if rpc_error_call is None:
        return None
      else:
        return _abortion_error(rpc_error_call)
    except grpc.FutureTimeoutError:
      raise future.TimeoutError()
    except grpc.FutureCancelledError:
      raise future.CancelledError()

  def traceback(self, timeout=None):
    try:
      return self._future.traceback(timeout=timeout)
    except grpc.FutureTimeoutError:
      raise future.TimeoutError()
    except grpc.FutureCancelledError:
      raise future.CancelledError()

  def add_done_callback(self, fn):
    self._future.add_done_callback(lambda ignored_callback: fn(self))

  def __iter__(self):
    return self

  def _next(self):
    try:
      return next(self._iterator)
    except grpc.RpcError as rpc_error_call:
      raise _abortion_error(rpc_error_call)

  def __next__(self):
    return self._next()

  def next(self):
    return self._next()

  def is_active(self):
    return self._call.is_active()

  def time_remaining(self):
    return self._call.time_remaining()

  def add_abortion_callback(self, abortion_callback):
    def done_callback():
      if self.code() is not grpc.StatusCode.OK:
        abortion_callback(_abortion(self._call))
    registered = self._call.add_callback(done_callback)
    return None if registered else done_callback()

  def protocol_context(self):
    return _InvocationProtocolContext()

  def initial_metadata(self):
    return self._call.initial_metadata()

  def terminal_metadata(self):
    return self._call.terminal_metadata()

  def code(self):
    return self._call.code()

  def details(self):
    return self._call.details()


def _blocking_unary_unary(
    channel, group, method, timeout, with_call, protocol_options, metadata,
    metadata_transformer, request, request_serializer, response_deserializer):
  try:
    multi_callable = channel.unary_unary(
        _common.fully_qualified_method(group, method),
        request_serializer=request_serializer,
        response_deserializer=response_deserializer)
    effective_metadata = _effective_metadata(metadata, metadata_transformer)
    if with_call:
      response, call = multi_callable.with_call(
          request, timeout=timeout, metadata=effective_metadata,
          credentials=_credentials(protocol_options))
      return response, _Rendezvous(None, None, call)
    else:
      return multi_callable(
          request, timeout=timeout, metadata=effective_metadata,
          credentials=_credentials(protocol_options))
  except grpc.RpcError as rpc_error_call:
    raise _abortion_error(rpc_error_call)


def _future_unary_unary(
    channel, group, method, timeout, protocol_options, metadata,
    metadata_transformer, request, request_serializer, response_deserializer):
  multi_callable = channel.unary_unary(
      _common.fully_qualified_method(group, method),
      request_serializer=request_serializer,
      response_deserializer=response_deserializer)
  effective_metadata = _effective_metadata(metadata, metadata_transformer)
  response_future = multi_callable.future(
      request, timeout=timeout, metadata=effective_metadata,
      credentials=_credentials(protocol_options))
  return _Rendezvous(response_future, None, response_future)


def _unary_stream(
    channel, group, method, timeout, protocol_options, metadata,
    metadata_transformer, request, request_serializer, response_deserializer):
  multi_callable = channel.unary_stream(
      _common.fully_qualified_method(group, method),
      request_serializer=request_serializer,
      response_deserializer=response_deserializer)
  effective_metadata = _effective_metadata(metadata, metadata_transformer)
  response_iterator = multi_callable(
      request, timeout=timeout, metadata=effective_metadata,
      credentials=_credentials(protocol_options))
  return _Rendezvous(None, response_iterator, response_iterator)


def _blocking_stream_unary(
    channel, group, method, timeout, with_call, protocol_options, metadata,
    metadata_transformer, request_iterator, request_serializer,
    response_deserializer):
  try:
    multi_callable = channel.stream_unary(
        _common.fully_qualified_method(group, method),
        request_serializer=request_serializer,
        response_deserializer=response_deserializer)
    effective_metadata = _effective_metadata(metadata, metadata_transformer)
    if with_call:
      response, call = multi_callable.with_call(
          request_iterator, timeout=timeout, metadata=effective_metadata,
          credentials=_credentials(protocol_options))
      return response, _Rendezvous(None, None, call)
    else:
      return multi_callable(
          request_iterator, timeout=timeout, metadata=effective_metadata,
          credentials=_credentials(protocol_options))
  except grpc.RpcError as rpc_error_call:
    raise _abortion_error(rpc_error_call)


def _future_stream_unary(
    channel, group, method, timeout, protocol_options, metadata,
    metadata_transformer, request_iterator, request_serializer,
    response_deserializer):
  multi_callable = channel.stream_unary(
      _common.fully_qualified_method(group, method),
      request_serializer=request_serializer,
      response_deserializer=response_deserializer)
  effective_metadata = _effective_metadata(metadata, metadata_transformer)
  response_future = multi_callable.future(
      request_iterator, timeout=timeout, metadata=effective_metadata,
      credentials=_credentials(protocol_options))
  return _Rendezvous(response_future, None, response_future)


def _stream_stream(
    channel, group, method, timeout, protocol_options, metadata,
    metadata_transformer, request_iterator, request_serializer,
    response_deserializer):
  multi_callable = channel.stream_stream(
      _common.fully_qualified_method(group, method),
      request_serializer=request_serializer,
      response_deserializer=response_deserializer)
  effective_metadata = _effective_metadata(metadata, metadata_transformer)
  response_iterator = multi_callable(
      request_iterator, timeout=timeout, metadata=effective_metadata,
      credentials=_credentials(protocol_options))
  return _Rendezvous(None, response_iterator, response_iterator)


class _UnaryUnaryMultiCallable(face.UnaryUnaryMultiCallable):

  def __init__(
      self, channel, group, method, metadata_transformer, request_serializer,
      response_deserializer):
    self._channel = channel
    self._group = group
    self._method = method
    self._metadata_transformer = metadata_transformer
    self._request_serializer = request_serializer
    self._response_deserializer = response_deserializer

  def __call__(
      self, request, timeout, metadata=None, with_call=False,
      protocol_options=None):
    return _blocking_unary_unary(
        self._channel, self._group, self._method, timeout, with_call,
        protocol_options, metadata, self._metadata_transformer, request,
        self._request_serializer, self._response_deserializer)

  def future(self, request, timeout, metadata=None, protocol_options=None):
    return _future_unary_unary(
        self._channel, self._group, self._method, timeout, protocol_options,
        metadata, self._metadata_transformer, request, self._request_serializer,
        self._response_deserializer)

  def event(
      self, request, receiver, abortion_callback, timeout,
      metadata=None, protocol_options=None):
    raise NotImplementedError()


class _UnaryStreamMultiCallable(face.UnaryStreamMultiCallable):

  def __init__(
      self, channel, group, method, metadata_transformer, request_serializer,
      response_deserializer):
    self._channel = channel
    self._group = group
    self._method = method
    self._metadata_transformer = metadata_transformer
    self._request_serializer = request_serializer
    self._response_deserializer = response_deserializer

  def __call__(self, request, timeout, metadata=None, protocol_options=None):
    return _unary_stream(
        self._channel, self._group, self._method, timeout, protocol_options,
        metadata, self._metadata_transformer, request, self._request_serializer,
        self._response_deserializer)

  def event(
      self, request, receiver, abortion_callback, timeout,
      metadata=None, protocol_options=None):
    raise NotImplementedError()


class _StreamUnaryMultiCallable(face.StreamUnaryMultiCallable):

  def __init__(
      self, channel, group, method, metadata_transformer, request_serializer,
      response_deserializer):
    self._channel = channel
    self._group = group
    self._method = method
    self._metadata_transformer = metadata_transformer
    self._request_serializer = request_serializer
    self._response_deserializer = response_deserializer

  def __call__(
      self, request_iterator, timeout, metadata=None, with_call=False,
      protocol_options=None):
    return _blocking_stream_unary(
        self._channel, self._group, self._method, timeout, with_call,
        protocol_options, metadata, self._metadata_transformer,
        request_iterator, self._request_serializer, self._response_deserializer)

  def future(
      self, request_iterator, timeout, metadata=None, protocol_options=None):
    return _future_stream_unary(
        self._channel, self._group, self._method, timeout, protocol_options,
        metadata, self._metadata_transformer, request_iterator,
        self._request_serializer, self._response_deserializer)

  def event(
      self, receiver, abortion_callback, timeout, metadata=None,
      protocol_options=None):
    raise NotImplementedError()


class _StreamStreamMultiCallable(face.StreamStreamMultiCallable):

  def __init__(
      self, channel, group, method, metadata_transformer, request_serializer,
      response_deserializer):
    self._channel = channel
    self._group = group
    self._method = method
    self._metadata_transformer = metadata_transformer
    self._request_serializer = request_serializer
    self._response_deserializer = response_deserializer

  def __call__(
      self, request_iterator, timeout, metadata=None, protocol_options=None):
    return _stream_stream(
        self._channel, self._group, self._method, timeout, protocol_options,
        metadata, self._metadata_transformer, request_iterator,
        self._request_serializer, self._response_deserializer)

  def event(
      self, receiver, abortion_callback, timeout, metadata=None,
      protocol_options=None):
    raise NotImplementedError()


class _GenericStub(face.GenericStub):

  def __init__(
      self, channel, metadata_transformer, request_serializers,
      response_deserializers):
    self._channel = channel
    self._metadata_transformer = metadata_transformer
    self._request_serializers = request_serializers or {}
    self._response_deserializers = response_deserializers or {}

  def blocking_unary_unary(
      self, group, method, request, timeout, metadata=None,
      with_call=None, protocol_options=None):
    request_serializer = self._request_serializers.get((group, method,))
    response_deserializer = self._response_deserializers.get((group, method,))
    return _blocking_unary_unary(
        self._channel, group, method, timeout, with_call, protocol_options,
        metadata, self._metadata_transformer, request, request_serializer,
        response_deserializer)

  def future_unary_unary(
      self, group, method, request, timeout, metadata=None,
      protocol_options=None):
    request_serializer = self._request_serializers.get((group, method,))
    response_deserializer = self._response_deserializers.get((group, method,))
    return _future_unary_unary(
        self._channel, group, method, timeout, protocol_options, metadata,
        self._metadata_transformer, request, request_serializer,
        response_deserializer)

  def inline_unary_stream(
      self, group, method, request, timeout, metadata=None,
      protocol_options=None):
    request_serializer = self._request_serializers.get((group, method,))
    response_deserializer = self._response_deserializers.get((group, method,))
    return _unary_stream(
        self._channel, group, method, timeout, protocol_options, metadata,
        self._metadata_transformer, request, request_serializer,
        response_deserializer)

  def blocking_stream_unary(
      self, group, method, request_iterator, timeout, metadata=None,
      with_call=None, protocol_options=None):
    request_serializer = self._request_serializers.get((group, method,))
    response_deserializer = self._response_deserializers.get((group, method,))
    return _blocking_stream_unary(
        self._channel, group, method, timeout, with_call, protocol_options,
        metadata, self._metadata_transformer, request_iterator,
        request_serializer, response_deserializer)

  def future_stream_unary(
      self, group, method, request_iterator, timeout, metadata=None,
      protocol_options=None):
    request_serializer = self._request_serializers.get((group, method,))
    response_deserializer = self._response_deserializers.get((group, method,))
    return _future_stream_unary(
        self._channel, group, method, timeout, protocol_options, metadata,
        self._metadata_transformer, request_iterator, request_serializer,
        response_deserializer)

  def inline_stream_stream(
      self, group, method, request_iterator, timeout, metadata=None,
      protocol_options=None):
    request_serializer = self._request_serializers.get((group, method,))
    response_deserializer = self._response_deserializers.get((group, method,))
    return _stream_stream(
        self._channel, group, method, timeout, protocol_options, metadata,
        self._metadata_transformer, request_iterator, request_serializer,
        response_deserializer)

  def event_unary_unary(
      self, group, method, request, receiver, abortion_callback, timeout,
      metadata=None, protocol_options=None):
    raise NotImplementedError()

  def event_unary_stream(
      self, group, method, request, receiver, abortion_callback, timeout,
      metadata=None, protocol_options=None):
    raise NotImplementedError()

  def event_stream_unary(
      self, group, method, receiver, abortion_callback, timeout,
      metadata=None, protocol_options=None):
    raise NotImplementedError()

  def event_stream_stream(
      self, group, method, receiver, abortion_callback, timeout,
      metadata=None, protocol_options=None):
    raise NotImplementedError()

  def unary_unary(self, group, method):
    request_serializer = self._request_serializers.get((group, method,))
    response_deserializer = self._response_deserializers.get((group, method,))
    return _UnaryUnaryMultiCallable(
        self._channel, group, method, self._metadata_transformer,
        request_serializer, response_deserializer)

  def unary_stream(self, group, method):
    request_serializer = self._request_serializers.get((group, method,))
    response_deserializer = self._response_deserializers.get((group, method,))
    return _UnaryStreamMultiCallable(
        self._channel, group, method, self._metadata_transformer,
        request_serializer, response_deserializer)

  def stream_unary(self, group, method):
    request_serializer = self._request_serializers.get((group, method,))
    response_deserializer = self._response_deserializers.get((group, method,))
    return _StreamUnaryMultiCallable(
        self._channel, group, method, self._metadata_transformer,
        request_serializer, response_deserializer)

  def stream_stream(self, group, method):
    request_serializer = self._request_serializers.get((group, method,))
    response_deserializer = self._response_deserializers.get((group, method,))
    return _StreamStreamMultiCallable(
        self._channel, group, method, self._metadata_transformer,
        request_serializer, response_deserializer)

  def __enter__(self):
    return self

  def __exit__(self, exc_type, exc_val, exc_tb):
    return False


class _DynamicStub(face.DynamicStub):

  def __init__(self, generic_stub, group, cardinalities):
    self._generic_stub = generic_stub
    self._group = group
    self._cardinalities = cardinalities

  def __getattr__(self, attr):
    method_cardinality = self._cardinalities.get(attr)
    if method_cardinality is cardinality.Cardinality.UNARY_UNARY:
      return self._generic_stub.unary_unary(self._group, attr)
    elif method_cardinality is cardinality.Cardinality.UNARY_STREAM:
      return self._generic_stub.unary_stream(self._group, attr)
    elif method_cardinality is cardinality.Cardinality.STREAM_UNARY:
      return self._generic_stub.stream_unary(self._group, attr)
    elif method_cardinality is cardinality.Cardinality.STREAM_STREAM:
      return self._generic_stub.stream_stream(self._group, attr)
    else:
      raise AttributeError('_DynamicStub object has no attribute "%s"!' % attr)

  def __enter__(self):
    return self

  def __exit__(self, exc_type, exc_val, exc_tb):
    return False


def generic_stub(
    channel, host, metadata_transformer, request_serializers,
    response_deserializers):
  return _GenericStub(
      channel, metadata_transformer, request_serializers,
      response_deserializers)


def dynamic_stub(
    channel, service, cardinalities, host, metadata_transformer,
    request_serializers, response_deserializers):
  return _DynamicStub(
      _GenericStub(
          channel, metadata_transformer, request_serializers,
          response_deserializers),
      service, cardinalities)



# Copyright 2016, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""GRPCAuthMetadataPlugins for standard authentication."""

import inspect
from concurrent import futures

import grpc


def _sign_request(callback, token, error):
  metadata = (('authorization', 'Bearer {}'.format(token)),)
  callback(metadata, error)


class GoogleCallCredentials(grpc.AuthMetadataPlugin):
  """Metadata wrapper for GoogleCredentials from the oauth2client library."""

  def __init__(self, credentials):
    self._credentials = credentials
    self._pool = futures.ThreadPoolExecutor(max_workers=1)

    # Hack to determine if these are JWT creds and we need to pass
    # additional_claims when getting a token
    if 'additional_claims' in inspect.getargspec(
        credentials.get_access_token).args:
      self._is_jwt = True
    else:
      self._is_jwt = False

  def __call__(self, context, callback):
    # MetadataPlugins cannot block (see grpc.beta.interfaces.py)
    if self._is_jwt:
      future = self._pool.submit(self._credentials.get_access_token,
                                 additional_claims={'aud': context.service_url})
    else:
      future = self._pool.submit(self._credentials.get_access_token)
    future.add_done_callback(lambda x: self._get_token_callback(callback, x))

  def _get_token_callback(self, callback, future):
    try:
      access_token = future.result().access_token
    except Exception as e:
      _sign_request(callback, None, e)
    else:
      _sign_request(callback, access_token, None)

  def __del__(self):
    self._pool.shutdown(wait=False)


class AccessTokenCallCredentials(grpc.AuthMetadataPlugin):
  """Metadata wrapper for raw access token credentials."""

  def __init__(self, access_token):
    self._access_token = access_token

  def __call__(self, context, callback):
    _sign_request(callback, self._access_token, None)



#!/usr/bin/env python

from __future__ import unicode_literals

'''Generate header file for nanopb from a ProtoBuf FileDescriptorSet.'''
nanopb_version = "nanopb-0.3.5-dev"

import sys
import re
from functools import reduce

try:
    # Add some dummy imports to keep packaging tools happy.
    import google, distutils.util # bbfreeze seems to need these
    import pkg_resources # pyinstaller / protobuf 2.5 seem to need these
except:
    # Don't care, we will error out later if it is actually important.
    pass

try:
    import google.protobuf.text_format as text_format
    import google.protobuf.descriptor_pb2 as descriptor
except:
    sys.stderr.write('''
         *************************************************************
         *** Could not import the Google protobuf Python libraries ***
         *** Try installing package 'python-protobuf' or similar.  ***
         *************************************************************
    ''' + '\n')
    raise

try:
    import proto.nanopb_pb2 as nanopb_pb2
    import proto.plugin_pb2 as plugin_pb2
except:
    sys.stderr.write('''
         ********************************************************************
         *** Failed to import the protocol definitions for generator.     ***
         *** You have to run 'make' in the nanopb/generator/proto folder. ***
         ********************************************************************
    ''' + '\n')
    raise

# ---------------------------------------------------------------------------
#                     Generation of single fields
# ---------------------------------------------------------------------------

import time
import os.path

# Values are tuple (c type, pb type, encoded size, int_size_allowed)
FieldD = descriptor.FieldDescriptorProto
datatypes = {
    FieldD.TYPE_BOOL:       ('bool',     'BOOL',        1,  False),
    FieldD.TYPE_DOUBLE:     ('double',   'DOUBLE',      8,  False),
    FieldD.TYPE_FIXED32:    ('uint32_t', 'FIXED32',     4,  False),
    FieldD.TYPE_FIXED64:    ('uint64_t', 'FIXED64',     8,  False),
    FieldD.TYPE_FLOAT:      ('float',    'FLOAT',       4,  False),
    FieldD.TYPE_INT32:      ('int32_t',  'INT32',      10,  True),
    FieldD.TYPE_INT64:      ('int64_t',  'INT64',      10,  True),
    FieldD.TYPE_SFIXED32:   ('int32_t',  'SFIXED32',    4,  False),
    FieldD.TYPE_SFIXED64:   ('int64_t',  'SFIXED64',    8,  False),
    FieldD.TYPE_SINT32:     ('int32_t',  'SINT32',      5,  True),
    FieldD.TYPE_SINT64:     ('int64_t',  'SINT64',     10,  True),
    FieldD.TYPE_UINT32:     ('uint32_t', 'UINT32',      5,  True),
    FieldD.TYPE_UINT64:     ('uint64_t', 'UINT64',     10,  True)
}

# Integer size overrides (from .proto settings)
intsizes = {
    nanopb_pb2.IS_8:     'int8_t',
    nanopb_pb2.IS_16:    'int16_t',
    nanopb_pb2.IS_32:    'int32_t',
    nanopb_pb2.IS_64:    'int64_t',
}

# String types (for python 2 / python 3 compatibility)
try:
    strtypes = (unicode, str)
except NameError:
    strtypes = (str, )

class Names:
    '''Keeps a set of nested names and formats them to C identifier.'''
    def __init__(self, parts = ()):
        if isinstance(parts, Names):
            parts = parts.parts
        self.parts = tuple(parts)

    def __str__(self):
        return '_'.join(self.parts)

    def __add__(self, other):
        if isinstance(other, strtypes):
            return Names(self.parts + (other,))
        elif isinstance(other, tuple):
            return Names(self.parts + other)
        else:
            raise ValueError("Name parts should be of type str")

    def __eq__(self, other):
        return isinstance(other, Names) and self.parts == other.parts

def names_from_type_name(type_name):
    '''Parse Names() from FieldDescriptorProto type_name'''
    if type_name[0] != '.':
        raise NotImplementedError("Lookup of non-absolute type names is not supported")
    return Names(type_name[1:].split('.'))

def varint_max_size(max_value):
    '''Returns the maximum number of bytes a varint can take when encoded.'''
    if max_value < 0:
        max_value = 2**64 - max_value
    for i in range(1, 11):
        if (max_value >> (i * 7)) == 0:
            return i
    raise ValueError("Value too large for varint: " + str(max_value))

assert varint_max_size(-1) == 10
assert varint_max_size(0) == 1
assert varint_max_size(127) == 1
assert varint_max_size(128) == 2

class EncodedSize:
    '''Class used to represent the encoded size of a field or a message.
    Consists of a combination of symbolic sizes and integer sizes.'''
    def __init__(self, value = 0, symbols = []):
        if isinstance(value, strtypes + (Names,)):
            symbols = [str(value)]
            value = 0
        self.value = value
        self.symbols = symbols

    def __add__(self, other):
        if isinstance(other, int):
            return EncodedSize(self.value + other, self.symbols)
        elif isinstance(other, strtypes + (Names,)):
            return EncodedSize(self.value, self.symbols + [str(other)])
        elif isinstance(other, EncodedSize):
            return EncodedSize(self.value + other.value, self.symbols + other.symbols)
        else:
            raise ValueError("Cannot add size: " + repr(other))

    def __mul__(self, other):
        if isinstance(other, int):
            return EncodedSize(self.value * other, [str(other) + '*' + s for s in self.symbols])
        else:
            raise ValueError("Cannot multiply size: " + repr(other))

    def __str__(self):
        if not self.symbols:
            return str(self.value)
        else:
            return '(' + str(self.value) + ' + ' + ' + '.join(self.symbols) + ')'

    def upperlimit(self):
        if not self.symbols:
            return self.value
        else:
            return 2**32 - 1

class Enum:
    def __init__(self, names, desc, enum_options):
        '''desc is EnumDescriptorProto'''

        self.options = enum_options
        self.names = names + desc.name

        if enum_options.long_names:
            self.values = [(self.names + x.name, x.number) for x in desc.value]
        else:
            self.values = [(names + x.name, x.number) for x in desc.value]

        self.value_longnames = [self.names + x.name for x in desc.value]
        self.packed = enum_options.packed_enum

    def has_negative(self):
        for n, v in self.values:
            if v < 0:
                return True
        return False

    def encoded_size(self):
        return max([varint_max_size(v) for n,v in self.values])

    def __str__(self):
        result = 'typedef enum _%s {\n' % self.names
        result += ',\n'.join(["    %s = %d" % x for x in self.values])
        result += '\n}'

        if self.packed:
            result += ' pb_packed'

        result += ' %s;' % self.names

        if not self.options.long_names:
            # Define the long names always so that enum value references
            # from other files work properly.
            for i, x in enumerate(self.values):
                result += '\n#define %s %s' % (self.value_longnames[i], x[0])

        return result

class FieldMaxSize:
    def __init__(self, worst = 0, checks = [], field_name = 'undefined'):
        if isinstance(worst, list):
            self.worst = max(i for i in worst if i is not None)
        else:
            self.worst = worst

        self.worst_field = field_name
        self.checks = checks

    def extend(self, extend, field_name = None):
        self.worst = max(self.worst, extend.worst)

        if self.worst == extend.worst:
            self.worst_field = extend.worst_field

        self.checks.extend(extend.checks)

class Field:
    def __init__(self, struct_name, desc, field_options):
        '''desc is FieldDescriptorProto'''
        self.tag = desc.number
        self.struct_name = struct_name
        self.union_name = None
        self.name = desc.name
        self.default = None
        self.max_size = None
        self.max_count = None
        self.array_decl = ""
        self.enc_size = None
        self.ctype = None

        # Parse field options
        if field_options.HasField("max_size"):
            self.max_size = field_options.max_size

        if field_options.HasField("max_count"):
            self.max_count = field_options.max_count

        if desc.HasField('default_value'):
            self.default = desc.default_value

        # Check field rules, i.e. required/optional/repeated.
        can_be_static = True
        if desc.label == FieldD.LABEL_REQUIRED:
            self.rules = 'REQUIRED'
        elif desc.label == FieldD.LABEL_OPTIONAL:
            self.rules = 'OPTIONAL'
        elif desc.label == FieldD.LABEL_REPEATED:
            self.rules = 'REPEATED'
            if self.max_count is None:
                can_be_static = False
            else:
                self.array_decl = '[%d]' % self.max_count
        else:
            raise NotImplementedError(desc.label)

        # Check if the field can be implemented with static allocation
        # i.e. whether the data size is known.
        if desc.type == FieldD.TYPE_STRING and self.max_size is None:
            can_be_static = False

        if desc.type == FieldD.TYPE_BYTES and self.max_size is None:
            can_be_static = False

        # Decide how the field data will be allocated
        if field_options.type == nanopb_pb2.FT_DEFAULT:
            if can_be_static:
                field_options.type = nanopb_pb2.FT_STATIC
            else:
                field_options.type = nanopb_pb2.FT_CALLBACK

        if field_options.type == nanopb_pb2.FT_STATIC and not can_be_static:
            raise Exception("Field %s is defined as static, but max_size or "
                            "max_count is not given." % self.name)

        if field_options.type == nanopb_pb2.FT_STATIC:
            self.allocation = 'STATIC'
        elif field_options.type == nanopb_pb2.FT_POINTER:
            self.allocation = 'POINTER'
        elif field_options.type == nanopb_pb2.FT_CALLBACK:
            self.allocation = 'CALLBACK'
        else:
            raise NotImplementedError(field_options.type)

        # Decide the C data type to use in the struct.
        if desc.type in datatypes:
            self.ctype, self.pbtype, self.enc_size, isa = datatypes[desc.type]

            # Override the field size if user wants to use smaller integers
            if isa and field_options.int_size != nanopb_pb2.IS_DEFAULT:
                self.ctype = intsizes[field_options.int_size]
                if desc.type == FieldD.TYPE_UINT32 or desc.type == FieldD.TYPE_UINT64:
                    self.ctype = 'u' + self.ctype;
        elif desc.type == FieldD.TYPE_ENUM:
            self.pbtype = 'ENUM'
            self.ctype = names_from_type_name(desc.type_name)
            if self.default is not None:
                self.default = self.ctype + self.default
            self.enc_size = None # Needs to be filled in when enum values are known
        elif desc.type == FieldD.TYPE_STRING:
            self.pbtype = 'STRING'
            self.ctype = 'char'
            if self.allocation == 'STATIC':
                self.ctype = 'char'
                self.array_decl += '[%d]' % self.max_size
                self.enc_size = varint_max_size(self.max_size) + self.max_size
        elif desc.type == FieldD.TYPE_BYTES:
            self.pbtype = 'BYTES'
            if self.allocation == 'STATIC':
                self.ctype = self.struct_name + self.name + 't'
                self.enc_size = varint_max_size(self.max_size) + self.max_size
            elif self.allocation == 'POINTER':
                self.ctype = 'pb_bytes_array_t'
        elif desc.type == FieldD.TYPE_MESSAGE:
            self.pbtype = 'MESSAGE'
            self.ctype = self.submsgname = names_from_type_name(desc.type_name)
            self.enc_size = None # Needs to be filled in after the message type is available
        else:
            raise NotImplementedError(desc.type)

    def __lt__(self, other):
        return self.tag < other.tag

    def __str__(self):
        result = ''
        if self.allocation == 'POINTER':
            if self.rules == 'REPEATED':
                result += '    pb_size_t ' + self.name + '_count;\n'

            if self.pbtype == 'MESSAGE':
                # Use struct definition, so recursive submessages are possible
                result += '    struct _%s *%s;' % (self.ctype, self.name)
            elif self.rules == 'REPEATED' and self.pbtype in ['STRING', 'BYTES']:
                # String/bytes arrays need to be defined as pointers to pointers
                result += '    %s **%s;' % (self.ctype, self.name)
            else:
                result += '    %s *%s;' % (self.ctype, self.name)
        elif self.allocation == 'CALLBACK':
            result += '    pb_callback_t %s;' % self.name
        else:
            if self.rules == 'OPTIONAL' and self.allocation == 'STATIC':
                result += '    bool has_' + self.name + ';\n'
            elif self.rules == 'REPEATED' and self.allocation == 'STATIC':
                result += '    pb_size_t ' + self.name + '_count;\n'
            result += '    %s %s%s;' % (self.ctype, self.name, self.array_decl)
        return result

    def types(self):
        '''Return definitions for any special types this field might need.'''
        if self.pbtype == 'BYTES' and self.allocation == 'STATIC':
            result = 'typedef PB_BYTES_ARRAY_T(%d) %s;\n' % (self.max_size, self.ctype)
        else:
            result = ''
        return result

    def get_dependencies(self):
        '''Get list of type names used by this field.'''
        if self.allocation == 'STATIC':
            return [str(self.ctype)]
        else:
            return []

    def get_initializer(self, null_init, inner_init_only = False):
        '''Return literal expression for this field's default value.
        null_init: If True, initialize to a 0 value instead of default from .proto
        inner_init_only: If True, exclude initialization for any count/has fields
        '''

        inner_init = None
        if self.pbtype == 'MESSAGE':
            if null_init:
                inner_init = '%s_init_zero' % self.ctype
            else:
                inner_init = '%s_init_default' % self.ctype
        elif self.default is None or null_init:
            if self.pbtype == 'STRING':
                inner_init = '""'
            elif self.pbtype == 'BYTES':
                inner_init = '{0, {0}}'
            elif self.pbtype in ('ENUM', 'UENUM'):
                inner_init = '(%s)0' % self.ctype
            else:
                inner_init = '0'
        else:
            if self.pbtype == 'STRING':
                inner_init = self.default.replace('"', '\\"')
                inner_init = '"' + inner_init + '"'
            elif self.pbtype == 'BYTES':
                data = ['0x%02x' % ord(c) for c in self.default]
                if len(data) == 0:
                    inner_init = '{0, {0}}'
                else:
                    inner_init = '{%d, {%s}}' % (len(data), ','.join(data))
            elif self.pbtype in ['FIXED32', 'UINT32']:
                inner_init = str(self.default) + 'u'
            elif self.pbtype in ['FIXED64', 'UINT64']:
                inner_init = str(self.default) + 'ull'
            elif self.pbtype in ['SFIXED64', 'INT64']:
                inner_init = str(self.default) + 'll'
            else:
                inner_init = str(self.default)

        if inner_init_only:
            return inner_init

        outer_init = None
        if self.allocation == 'STATIC':
            if self.rules == 'REPEATED':
                outer_init = '0, {'
                outer_init += ', '.join([inner_init] * self.max_count)
                outer_init += '}'
            elif self.rules == 'OPTIONAL':
                outer_init = 'false, ' + inner_init
            else:
                outer_init = inner_init
        elif self.allocation == 'POINTER':
            if self.rules == 'REPEATED':
                outer_init = '0, NULL'
            else:
                outer_init = 'NULL'
        elif self.allocation == 'CALLBACK':
            if self.pbtype == 'EXTENSION':
                outer_init = 'NULL'
            else:
                outer_init = '{{NULL}, NULL}'

        return outer_init

    def default_decl(self, declaration_only = False):
        '''Return definition for this field's default value.'''
        if self.default is None:
            return None

        ctype = self.ctype
        default = self.get_initializer(False, True)
        array_decl = ''

        if self.pbtype == 'STRING':
            if self.allocation != 'STATIC':
                return None # Not implemented
            array_decl = '[%d]' % self.max_size
        elif self.pbtype == 'BYTES':
            if self.allocation != 'STATIC':
                return None # Not implemented

        if declaration_only:
            return 'extern const %s %s_default%s;' % (ctype, self.struct_name + self.name, array_decl)
        else:
            return 'const %s %s_default%s = %s;' % (ctype, self.struct_name + self.name, array_decl, default)

    def tags(self):
        '''Return the #define for the tag number of this field.'''
        identifier = '%s_%s_tag' % (self.struct_name, self.name)
        return '#define %-40s %d\n' % (identifier, self.tag)

    def pb_field_t(self, prev_field_name):
        '''Return the pb_field_t initializer to use in the constant array.
        prev_field_name is the name of the previous field or None.
        '''

        if self.rules == 'ONEOF':
            result = '    PB_ONEOF_FIELD(%s, ' % self.union_name
        else:
            result = '    PB_FIELD('

        result += '%3d, ' % self.tag
        result += '%-8s, ' % self.pbtype
        result += '%s, ' % self.rules
        result += '%-8s, ' % self.allocation
        result += '%s, ' % ("FIRST" if not prev_field_name else "OTHER")
        result += '%s, ' % self.struct_name
        result += '%s, ' % self.name
        result += '%s, ' % (prev_field_name or self.name)

        if self.pbtype == 'MESSAGE':
            result += '&%s_fields)' % self.submsgname
        elif self.default is None:
            result += '0)'
        elif self.pbtype in ['BYTES', 'STRING'] and self.allocation != 'STATIC':
            result += '0)' # Arbitrary size default values not implemented
        elif self.rules == 'OPTEXT':
            result += '0)' # Default value for extensions is not implemented
        else:
            result += '&%s_default)' % (self.struct_name + self.name)

        return result

    def largest_field_value(self):
        '''Determine if this field needs 16bit or 32bit pb_field_t structure to compile properly.
        Returns numeric value or a C-expression for assert.'''
        check = []
        if self.pbtype == 'MESSAGE':
            if self.rules == 'REPEATED' and self.allocation == 'STATIC':
                check.append('pb_membersize(%s, %s[0])' % (self.struct_name, self.name))
            elif self.rules == 'ONEOF':
                check.append('pb_membersize(%s, %s.%s)' % (self.struct_name, self.union_name, self.name))
            else:
                check.append('pb_membersize(%s, %s)' % (self.struct_name, self.name))

        return FieldMaxSize([self.tag, self.max_size, self.max_count],
                            check,
                            ('%s.%s' % (self.struct_name, self.name)))

    def encoded_size(self, dependencies):
        '''Return the maximum size that this field can take when encoded,
        including the field tag. If the size cannot be determined, returns
        None.'''

        if self.allocation != 'STATIC':
            return None

        if self.pbtype == 'MESSAGE':
            encsize = None
            if str(self.submsgname) in dependencies:
                submsg = dependencies[str(self.submsgname)]
                encsize = submsg.encoded_size(dependencies)
                if encsize is not None:
                    # Include submessage length prefix
                    encsize += varint_max_size(encsize.upperlimit())

            if encsize is None:
                # Submessage or its size cannot be found.
                # This can occur if submessage is defined in different
                # file, and it or its .options could not be found.
                # Instead of direct numeric value, reference the size that
                # has been #defined in the other file.
                encsize = EncodedSize(self.submsgname + 'size')

                # We will have to make a conservative assumption on the length
                # prefix size, though.
                encsize += 5

        elif self.pbtype in ['ENUM', 'UENUM']:
            if str(self.ctype) in dependencies:
                enumtype = dependencies[str(self.ctype)]
                encsize = enumtype.encoded_size()
            else:
                # Conservative assumption
                encsize = 10

        elif self.enc_size is None:
            raise RuntimeError("Could not determine encoded size for %s.%s"
                               % (self.struct_name, self.name))
        else:
            encsize = EncodedSize(self.enc_size)

        encsize += varint_max_size(self.tag << 3) # Tag + wire type

        if self.rules == 'REPEATED':
            # Decoders must be always able to handle unpacked arrays.
            # Therefore we have to reserve space for it, even though
            # we emit packed arrays ourselves.
            encsize *= self.max_count

        return encsize


class ExtensionRange(Field):
    def __init__(self, struct_name, range_start, field_options):
        '''Implements a special pb_extension_t* field in an extensible message
        structure. The range_start signifies the index at which the extensions
        start. Not necessarily all tags above this are extensions, it is merely
        a speed optimization.
        '''
        self.tag = range_start
        self.struct_name = struct_name
        self.name = 'extensions'
        self.pbtype = 'EXTENSION'
        self.rules = 'OPTIONAL'
        self.allocation = 'CALLBACK'
        self.ctype = 'pb_extension_t'
        self.array_decl = ''
        self.default = None
        self.max_size = 0
        self.max_count = 0

    def __str__(self):
        return '    pb_extension_t *extensions;'

    def types(self):
        return ''

    def tags(self):
        return ''

    def encoded_size(self, dependencies):
        # We exclude extensions from the count, because they cannot be known
        # until runtime. Other option would be to return None here, but this
        # way the value remains useful if extensions are not used.
        return EncodedSize(0)

class ExtensionField(Field):
    def __init__(self, struct_name, desc, field_options):
        self.fullname = struct_name + desc.name
        self.extendee_name = names_from_type_name(desc.extendee)
        Field.__init__(self, self.fullname + 'struct', desc, field_options)

        if self.rules != 'OPTIONAL':
            self.skip = True
        else:
            self.skip = False
            self.rules = 'OPTEXT'

    def tags(self):
        '''Return the #define for the tag number of this field.'''
        identifier = '%s_tag' % self.fullname
        return '#define %-40s %d\n' % (identifier, self.tag)

    def extension_decl(self):
        '''Declaration of the extension type in the .pb.h file'''
        if self.skip:
            msg = '/* Extension field %s was skipped because only "optional"\n' % self.fullname
            msg +='   type of extension fields is currently supported. */\n'
            return msg

        return ('extern const pb_extension_type_t %s; /* field type: %s */\n' %
            (self.fullname, str(self).strip()))

    def extension_def(self):
        '''Definition of the extension type in the .pb.c file'''

        if self.skip:
            return ''

        result  = 'typedef struct {\n'
        result += str(self)
        result += '\n} %s;\n\n' % self.struct_name
        result += ('static const pb_field_t %s_field = \n  %s;\n\n' %
                    (self.fullname, self.pb_field_t(None)))
        result += 'const pb_extension_type_t %s = {\n' % self.fullname
        result += '    NULL,\n'
        result += '    NULL,\n'
        result += '    &%s_field\n' % self.fullname
        result += '};\n'
        return result


# ---------------------------------------------------------------------------
#                   Generation of oneofs (unions)
# ---------------------------------------------------------------------------

class OneOf(Field):
    def __init__(self, struct_name, oneof_desc):
        self.struct_name = struct_name
        self.name = oneof_desc.name
        self.ctype = 'union'
        self.pbtype = 'oneof'
        self.fields = []
        self.allocation = 'ONEOF'
        self.default = None
        self.rules = 'ONEOF'

    def add_field(self, field):
        if field.allocation == 'CALLBACK':
            raise Exception("Callback fields inside of oneof are not supported"
                            + " (field %s)" % field.name)

        field.union_name = self.name
        field.rules = 'ONEOF'
        self.fields.append(field)
        self.fields.sort(key = lambda f: f.tag)

        # Sort by the lowest tag number inside union
        self.tag = min([f.tag for f in self.fields])

    def __str__(self):
        result = ''
        if self.fields:
            result += '    pb_size_t which_' + self.name + ";\n"
            result += '    union {\n'
            for f in self.fields:
                result += '    ' + str(f).replace('\n', '\n    ') + '\n'
            result += '    } ' + self.name + ';'
        return result

    def types(self):
        return ''.join([f.types() for f in self.fields])

    def get_dependencies(self):
        deps = []
        for f in self.fields:
            deps += f.get_dependencies()
        return deps

    def get_initializer(self, null_init):
        return '0, {' + self.fields[0].get_initializer(null_init) + '}'

    def default_decl(self, declaration_only = False):
        return None

    def tags(self):
        return '\n'.join([f.tags() for f in self.fields])

    def pb_field_t(self, prev_field_name):
        result = ',\n'.join([f.pb_field_t(prev_field_name) for f in self.fields])
        return result

    def largest_field_value(self):
        largest = FieldMaxSize()
        for f in self.fields:
            largest.extend(f.largest_field_value())
        return largest

    def encoded_size(self, dependencies):
        largest = EncodedSize(0)
        for f in self.fields:
            size = f.encoded_size(dependencies)
            if size is None:
                return None
            elif size.symbols:
                return None # Cannot resolve maximum of symbols
            elif size.value > largest.value:
                largest = size

        return largest

# ---------------------------------------------------------------------------
#                   Generation of messages (structures)
# ---------------------------------------------------------------------------


class Message:
    def __init__(self, names, desc, message_options):
        self.name = names
        self.fields = []
        self.oneofs = {}
        no_unions = []

        if message_options.msgid:
            self.msgid = message_options.msgid

        if hasattr(desc, 'oneof_decl'):
            for i, f in enumerate(desc.oneof_decl):
                oneof_options = get_nanopb_suboptions(desc, message_options, self.name + f.name)
                if oneof_options.no_unions:
                    no_unions.append(i) # No union, but add fields normally
                elif oneof_options.type == nanopb_pb2.FT_IGNORE:
                    pass # No union and skip fields also
                else:
                    oneof = OneOf(self.name, f)
                    self.oneofs[i] = oneof
                    self.fields.append(oneof)

        for f in desc.field:
            field_options = get_nanopb_suboptions(f, message_options, self.name + f.name)
            if field_options.type == nanopb_pb2.FT_IGNORE:
                continue

            field = Field(self.name, f, field_options)
            if (hasattr(f, 'oneof_index') and
                f.HasField('oneof_index') and
                f.oneof_index not in no_unions):
                if f.oneof_index in self.oneofs:
                    self.oneofs[f.oneof_index].add_field(field)
            else:
                self.fields.append(field)

        if len(desc.extension_range) > 0:
            field_options = get_nanopb_suboptions(desc, message_options, self.name + 'extensions')
            range_start = min([r.start for r in desc.extension_range])
            if field_options.type != nanopb_pb2.FT_IGNORE:
                self.fields.append(ExtensionRange(self.name, range_start, field_options))

        self.packed = message_options.packed_struct
        self.ordered_fields = self.fields[:]
        self.ordered_fields.sort()

    def get_dependencies(self):
        '''Get list of type names that this structure refers to.'''
        deps = []
        for f in self.fields:
            deps += f.get_dependencies()
        return deps

    def __str__(self):
        result = 'typedef struct _%s {\n' % self.name

        if not self.ordered_fields:
            # Empty structs are not allowed in C standard.
            # Therefore add a dummy field if an empty message occurs.
            result += '    uint8_t dummy_field;'

        result += '\n'.join([str(f) for f in self.ordered_fields])
        result += '\n}'

        if self.packed:
            result += ' pb_packed'

        result += ' %s;' % self.name

        if self.packed:
            result = 'PB_PACKED_STRUCT_START\n' + result
            result += '\nPB_PACKED_STRUCT_END'

        return result

    def types(self):
        return ''.join([f.types() for f in self.fields])

    def get_initializer(self, null_init):
        if not self.ordered_fields:
            return '{0}'

        parts = []
        for field in self.ordered_fields:
            parts.append(field.get_initializer(null_init))
        return '{' + ', '.join(parts) + '}'

    def default_decl(self, declaration_only = False):
        result = ""
        for field in self.fields:
            default = field.default_decl(declaration_only)
            if default is not None:
                result += default + '\n'
        return result

    def count_required_fields(self):
        '''Returns number of required fields inside this message'''
        count = 0
        for f in self.fields:
            if not isinstance(f, OneOf):
                if f.rules == 'REQUIRED':
                    count += 1
        return count

    def count_all_fields(self):
        count = 0
        for f in self.fields:
            if isinstance(f, OneOf):
                count += len(f.fields)
            else:
                count += 1
        return count

    def fields_declaration(self):
        result = 'extern const pb_field_t %s_fields[%d];' % (self.name, self.count_all_fields() + 1)
        return result

    def fields_definition(self):
        result = 'const pb_field_t %s_fields[%d] = {\n' % (self.name, self.count_all_fields() + 1)

        prev = None
        for field in self.ordered_fields:
            result += field.pb_field_t(prev)
            result += ',\n'
            if isinstance(field, OneOf):
                prev = field.name + '.' + field.fields[-1].name
            else:
                prev = field.name

        result += '    PB_LAST_FIELD\n};'
        return result

    def encoded_size(self, dependencies):
        '''Return the maximum size that this message can take when encoded.
        If the size cannot be determined, returns None.
        '''
        size = EncodedSize(0)
        for field in self.fields:
            fsize = field.encoded_size(dependencies)
            if fsize is None:
                return None
            size += fsize

        return size


# ---------------------------------------------------------------------------
#                    Processing of entire .proto files
# ---------------------------------------------------------------------------

def iterate_messages(desc, names = Names()):
    '''Recursively find all messages. For each, yield name, DescriptorProto.'''
    if hasattr(desc, 'message_type'):
        submsgs = desc.message_type
    else:
        submsgs = desc.nested_type

    for submsg in submsgs:
        sub_names = names + submsg.name
        yield sub_names, submsg

        for x in iterate_messages(submsg, sub_names):
            yield x

def iterate_extensions(desc, names = Names()):
    '''Recursively find all extensions.
    For each, yield name, FieldDescriptorProto.
    '''
    for extension in desc.extension:
        yield names, extension

    for subname, subdesc in iterate_messages(desc, names):
        for extension in subdesc.extension:
            yield subname, extension

def toposort2(data):
    '''Topological sort.
    From http://code.activestate.com/recipes/577413-topological-sort/
    This function is under the MIT license.
    '''
    for k, v in list(data.items()):
        v.discard(k) # Ignore self dependencies
    extra_items_in_deps = reduce(set.union, list(data.values()), set()) - set(data.keys())
    data.update(dict([(item, set()) for item in extra_items_in_deps]))
    while True:
        ordered = set(item for item,dep in list(data.items()) if not dep)
        if not ordered:
            break
        for item in sorted(ordered):
            yield item
        data = dict([(item, (dep - ordered)) for item,dep in list(data.items())
                if item not in ordered])
    assert not data, "A cyclic dependency exists amongst %r" % data

def sort_dependencies(messages):
    '''Sort a list of Messages based on dependencies.'''
    dependencies = {}
    message_by_name = {}
    for message in messages:
        dependencies[str(message.name)] = set(message.get_dependencies())
        message_by_name[str(message.name)] = message

    for msgname in toposort2(dependencies):
        if msgname in message_by_name:
            yield message_by_name[msgname]

def make_identifier(headername):
    '''Make #ifndef identifier that contains uppercase A-Z and digits 0-9'''
    result = ""
    for c in headername.upper():
        if c.isalnum():
            result += c
        else:
            result += '_'
    return result

class ProtoFile:
    def __init__(self, fdesc, file_options):
        '''Takes a FileDescriptorProto and parses it.'''
        self.fdesc = fdesc
        self.file_options = file_options
        self.dependencies = {}
        self.parse()

        # Some of types used in this file probably come from the file itself.
        # Thus it has implicit dependency on itself.
        self.add_dependency(self)

    def parse(self):
        self.enums = []
        self.messages = []
        self.extensions = []

        if self.fdesc.package:
            base_name = Names(self.fdesc.package.split('.'))
        else:
            base_name = Names()

        for enum in self.fdesc.enum_type:
            enum_options = get_nanopb_suboptions(enum, self.file_options, base_name + enum.name)
            self.enums.append(Enum(base_name, enum, enum_options))

        for names, message in iterate_messages(self.fdesc, base_name):
            message_options = get_nanopb_suboptions(message, self.file_options, names)

            if message_options.skip_message:
                continue

            self.messages.append(Message(names, message, message_options))
            for enum in message.enum_type:
                enum_options = get_nanopb_suboptions(enum, message_options, names + enum.name)
                self.enums.append(Enum(names, enum, enum_options))

        for names, extension in iterate_extensions(self.fdesc, base_name):
            field_options = get_nanopb_suboptions(extension, self.file_options, names + extension.name)
            if field_options.type != nanopb_pb2.FT_IGNORE:
                self.extensions.append(ExtensionField(names, extension, field_options))

    def add_dependency(self, other):
        for enum in other.enums:
            self.dependencies[str(enum.names)] = enum

        for msg in other.messages:
            self.dependencies[str(msg.name)] = msg

        # Fix field default values where enum short names are used.
        for enum in other.enums:
            if not enum.options.long_names:
                for message in self.messages:
                    for field in message.fields:
                        if field.default in enum.value_longnames:
                            idx = enum.value_longnames.index(field.default)
                            field.default = enum.values[idx][0]

        # Fix field data types where enums have negative values.
        for enum in other.enums:
            if not enum.has_negative():
                for message in self.messages:
                    for field in message.fields:
                        if field.pbtype == 'ENUM' and field.ctype == enum.names:
                            field.pbtype = 'UENUM'

    def generate_header(self, includes, headername, options):
        '''Generate content for a header file.
        Generates strings, which should be concatenated and stored to file.
        '''

        yield '/* Automatically generated nanopb header */\n'
        if options.notimestamp:
            yield '/* Generated by %s */\n\n' % (nanopb_version)
        else:
            yield '/* Generated by %s at %s. */\n\n' % (nanopb_version, time.asctime())

        symbol = make_identifier(headername)
        yield '#ifndef PB_%s_INCLUDED\n' % symbol
        yield '#define PB_%s_INCLUDED\n' % symbol
        try:
            yield options.libformat % ('pb.h')
        except TypeError:
            # no %s specified - use whatever was passed in as options.libformat
            yield options.libformat
        yield '\n'

        for incfile in includes:
            noext = os.path.splitext(incfile)[0]
            yield options.genformat % (noext + options.extension + '.h')
            yield '\n'

        yield '#if PB_PROTO_HEADER_VERSION != 30\n'
        yield '#error Regenerate this file with the current version of nanopb generator.\n'
        yield '#endif\n'
        yield '\n'

        yield '#ifdef __cplusplus\n'
        yield 'extern "C" {\n'
        yield '#endif\n\n'

        if self.enums:
            yield '/* Enum definitions */\n'
            for enum in self.enums:
                yield str(enum) + '\n\n'

        if self.messages:
            yield '/* Struct definitions */\n'
            for msg in sort_dependencies(self.messages):
                yield msg.types()
                yield str(msg) + '\n\n'

        if self.extensions:
            yield '/* Extensions */\n'
            for extension in self.extensions:
                yield extension.extension_decl()
            yield '\n'

        if self.messages:
            yield '/* Default values for struct fields */\n'
            for msg in self.messages:
                yield msg.default_decl(True)
            yield '\n'

            yield '/* Initializer values for message structs */\n'
            for msg in self.messages:
                identifier = '%s_init_default' % msg.name
                yield '#define %-40s %s\n' % (identifier, msg.get_initializer(False))
            for msg in self.messages:
                identifier = '%s_init_zero' % msg.name
                yield '#define %-40s %s\n' % (identifier, msg.get_initializer(True))
            yield '\n'

            yield '/* Field tags (for use in manual encoding/decoding) */\n'
            for msg in sort_dependencies(self.messages):
                for field in msg.fields:
                    yield field.tags()
            for extension in self.extensions:
                yield extension.tags()
            yield '\n'

            yield '/* Struct field encoding specification for nanopb */\n'
            for msg in self.messages:
                yield msg.fields_declaration() + '\n'
            yield '\n'

            yield '/* Maximum encoded size of messages (where known) */\n'
            for msg in self.messages:
                msize = msg.encoded_size(self.dependencies)
                if msize is not None:
                    identifier = '%s_size' % msg.name
                    yield '#define %-40s %s\n' % (identifier, msize)
            yield '\n'

            yield '/* Message IDs (where set with "msgid" option) */\n'

            yield '#ifdef PB_MSGID\n'
            for msg in self.messages:
                if hasattr(msg,'msgid'):
                    yield '#define PB_MSG_%d %s\n' % (msg.msgid, msg.name)
            yield '\n'

            symbol = make_identifier(headername.split('.')[0])
            yield '#define %s_MESSAGES \\\n' % symbol

            for msg in self.messages:
                m = "-1"
                msize = msg.encoded_size(self.dependencies)
                if msize is not None:
                    m = msize
                if hasattr(msg,'msgid'):
                    yield '\tPB_MSG(%d,%s,%s) \\\n' % (msg.msgid, m, msg.name)
            yield '\n'

            for msg in self.messages:
                if hasattr(msg,'msgid'):
                    yield '#define %s_msgid %d\n' % (msg.name, msg.msgid)
            yield '\n'

            yield '#endif\n\n'

        yield '#ifdef __cplusplus\n'
        yield '} /* extern "C" */\n'
        yield '#endif\n'

        # End of header
        yield '\n#endif\n'

    def generate_source(self, headername, options):
        '''Generate content for a source file.'''

        yield '/* Automatically generated nanopb constant definitions */\n'
        if options.notimestamp:
            yield '/* Generated by %s */\n\n' % (nanopb_version)
        else:
            yield '/* Generated by %s at %s. */\n\n' % (nanopb_version, time.asctime())
        yield options.genformat % (headername)
        yield '\n'

        yield '#if PB_PROTO_HEADER_VERSION != 30\n'
        yield '#error Regenerate this file with the current version of nanopb generator.\n'
        yield '#endif\n'
        yield '\n'

        for msg in self.messages:
            yield msg.default_decl(False)

        yield '\n\n'

        for msg in self.messages:
            yield msg.fields_definition() + '\n\n'

        for ext in self.extensions:
            yield ext.extension_def() + '\n'

        # Add checks for numeric limits
        if self.messages:
            largest_msg = max(self.messages, key = lambda m: m.count_required_fields())
            largest_count = largest_msg.count_required_fields()
            if largest_count > 64:
                yield '\n/* Check that missing required fields will be properly detected */\n'
                yield '#if PB_MAX_REQUIRED_FIELDS < %d\n' % largest_count
                yield '#error Properly detecting missing required fields in %s requires \\\n' % largest_msg.name
                yield '       setting PB_MAX_REQUIRED_FIELDS to %d or more.\n' % largest_count
                yield '#endif\n'

        max_field = FieldMaxSize()
        checks_msgnames = []
        for msg in self.messages:
            checks_msgnames.append(msg.name)
            for field in msg.fields:
                max_field.extend(field.largest_field_value())

        worst = max_field.worst
        worst_field = max_field.worst_field
        checks = max_field.checks

        if worst > 255 or checks:
            yield '\n/* Check that field information fits in pb_field_t */\n'

            if worst > 65535 or checks:
                yield '#if !defined(PB_FIELD_32BIT)\n'
                if worst > 65535:
                    yield '#error Field descriptor for %s is too large. Define PB_FIELD_32BIT to fix this.\n' % worst_field
                else:
                    assertion = ' && '.join(str(c) + ' < 65536' for c in checks)
                    msgs = '_'.join(str(n) for n in checks_msgnames)
                    yield '/* If you get an error here, it means that you need to define PB_FIELD_32BIT\n'
                    yield ' * compile-time option. You can do that in pb.h or on compiler command line.\n'
                    yield ' * \n'
                    yield ' * The reason you need to do this is that some of your messages contain tag\n'
                    yield ' * numbers or field sizes that are larger than what can fit in 8 or 16 bit\n'
                    yield ' * field descriptors.\n'
                    yield ' */\n'
                    yield 'PB_STATIC_ASSERT((%s), YOU_MUST_DEFINE_PB_FIELD_32BIT_FOR_MESSAGES_%s)\n'%(assertion,msgs)
                yield '#endif\n\n'

            if worst < 65536:
                yield '#if !defined(PB_FIELD_16BIT) && !defined(PB_FIELD_32BIT)\n'
                if worst > 255:
                    yield '#error Field descriptor for %s is too large. Define PB_FIELD_16BIT to fix this.\n' % worst_field
                else:
                    assertion = ' && '.join(str(c) + ' < 256' for c in checks)
                    msgs = '_'.join(str(n) for n in checks_msgnames)
                    yield '/* If you get an error here, it means that you need to define PB_FIELD_16BIT\n'
                    yield ' * compile-time option. You can do that in pb.h or on compiler command line.\n'
                    yield ' * \n'
                    yield ' * The reason you need to do this is that some of your messages contain tag\n'
                    yield ' * numbers or field sizes that are larger than what can fit in the default\n'
                    yield ' * 8 bit descriptors.\n'
                    yield ' */\n'
                    yield 'PB_STATIC_ASSERT((%s), YOU_MUST_DEFINE_PB_FIELD_16BIT_FOR_MESSAGES_%s)\n'%(assertion,msgs)
                yield '#endif\n\n'

        # Add check for sizeof(double)
        has_double = False
        for msg in self.messages:
            for field in msg.fields:
                if field.ctype == 'double':
                    has_double = True

        if has_double:
            yield '\n'
            yield '/* On some platforms (such as AVR), double is really float.\n'
            yield ' * These are not directly supported by nanopb, but see example_avr_double.\n'
            yield ' * To get rid of this error, remove any double fields from your .proto.\n'
            yield ' */\n'
            yield 'PB_STATIC_ASSERT(sizeof(double) == 8, DOUBLE_MUST_BE_8_BYTES)\n'

        yield '\n'

# ---------------------------------------------------------------------------
#                    Options parsing for the .proto files
# ---------------------------------------------------------------------------

from fnmatch import fnmatch

def read_options_file(infile):
    '''Parse a separate options file to list:
        [(namemask, options), ...]
    '''
    results = []
    data = infile.read()
    data = re.sub('/\*.*?\*/', '', data, flags = re.MULTILINE)
    data = re.sub('//.*?$', '', data, flags = re.MULTILINE)
    data = re.sub('#.*?$', '', data, flags = re.MULTILINE)
    for i, line in enumerate(data.split('\n')):
        line = line.strip()
        if not line:
            continue

        parts = line.split(None, 1)

        if len(parts) < 2:
            sys.stderr.write("%s:%d: " % (infile.name, i + 1) +
                             "Option lines should have space between field name and options. " +
                             "Skipping line: '%s'\n" % line)
            continue

        opts = nanopb_pb2.NanoPBOptions()

        try:
            text_format.Merge(parts[1], opts)
        except Exception as e:
            sys.stderr.write("%s:%d: " % (infile.name, i + 1) +
                             "Unparseable option line: '%s'. " % line +
                             "Error: %s\n" % str(e))
            continue
        results.append((parts[0], opts))

    return results

class Globals:
    '''Ugly global variables, should find a good way to pass these.'''
    verbose_options = False
    separate_options = []
    matched_namemasks = set()

def get_nanopb_suboptions(subdesc, options, name):
    '''Get copy of options, and merge information from subdesc.'''
    new_options = nanopb_pb2.NanoPBOptions()
    new_options.CopyFrom(options)

    # Handle options defined in a separate file
    dotname = '.'.join(name.parts)
    for namemask, options in Globals.separate_options:
        if fnmatch(dotname, namemask):
            Globals.matched_namemasks.add(namemask)
            new_options.MergeFrom(options)

    # Handle options defined in .proto
    if isinstance(subdesc.options, descriptor.FieldOptions):
        ext_type = nanopb_pb2.nanopb
    elif isinstance(subdesc.options, descriptor.FileOptions):
        ext_type = nanopb_pb2.nanopb_fileopt
    elif isinstance(subdesc.options, descriptor.MessageOptions):
        ext_type = nanopb_pb2.nanopb_msgopt
    elif isinstance(subdesc.options, descriptor.EnumOptions):
        ext_type = nanopb_pb2.nanopb_enumopt
    else:
        raise Exception("Unknown options type")

    if subdesc.options.HasExtension(ext_type):
        ext = subdesc.options.Extensions[ext_type]
        new_options.MergeFrom(ext)

    if Globals.verbose_options:
        sys.stderr.write("Options for " + dotname + ": ")
        sys.stderr.write(text_format.MessageToString(new_options) + "\n")

    return new_options


# ---------------------------------------------------------------------------
#                         Command line interface
# ---------------------------------------------------------------------------

import sys
import os.path
from optparse import OptionParser

optparser = OptionParser(
    usage = "Usage: nanopb_generator.py [options] file.pb ...",
    epilog = "Compile file.pb from file.proto by: 'protoc -ofile.pb file.proto'. " +
             "Output will be written to file.pb.h and file.pb.c.")
optparser.add_option("-x", dest="exclude", metavar="FILE", action="append", default=[],
    help="Exclude file from generated #include list.")
optparser.add_option("-e", "--extension", dest="extension", metavar="EXTENSION", default=".pb",
    help="Set extension to use instead of '.pb' for generated files. [default: %default]")
optparser.add_option("-f", "--options-file", dest="options_file", metavar="FILE", default="%s.options",
    help="Set name of a separate generator options file.")
optparser.add_option("-I", "--options-path", dest="options_path", metavar="DIR",
    action="append", default = [],
    help="Search for .options files additionally in this path")
optparser.add_option("-Q", "--generated-include-format", dest="genformat",
    metavar="FORMAT", default='#include "%s"\n',
    help="Set format string to use for including other .pb.h files. [default: %default]")
optparser.add_option("-L", "--library-include-format", dest="libformat",
    metavar="FORMAT", default='#include <%s>\n',
    help="Set format string to use for including the nanopb pb.h header. [default: %default]")
optparser.add_option("-T", "--no-timestamp", dest="notimestamp", action="store_true", default=False,
    help="Don't add timestamp to .pb.h and .pb.c preambles")
optparser.add_option("-q", "--quiet", dest="quiet", action="store_true", default=False,
    help="Don't print anything except errors.")
optparser.add_option("-v", "--verbose", dest="verbose", action="store_true", default=False,
    help="Print more information.")
optparser.add_option("-s", dest="settings", metavar="OPTION:VALUE", action="append", default=[],
    help="Set generator option (max_size, max_count etc.).")

def parse_file(filename, fdesc, options):
    '''Parse a single file. Returns a ProtoFile instance.'''
    toplevel_options = nanopb_pb2.NanoPBOptions()
    for s in options.settings:
        text_format.Merge(s, toplevel_options)

    if not fdesc:
        data = open(filename, 'rb').read()
        fdesc = descriptor.FileDescriptorSet.FromString(data).file[0]

    # Check if there is a separate .options file
    had_abspath = False
    try:
        optfilename = options.options_file % os.path.splitext(filename)[0]
    except TypeError:
        # No %s specified, use the filename as-is
        optfilename = options.options_file
        had_abspath = True

    paths = ['.'] + options.options_path
    for p in paths:
        if os.path.isfile(os.path.join(p, optfilename)):
            optfilename = os.path.join(p, optfilename)
            if options.verbose:
                sys.stderr.write('Reading options from ' + optfilename + '\n')
            Globals.separate_options = read_options_file(open(optfilename, "rU"))
            break
    else:
        # If we are given a full filename and it does not exist, give an error.
        # However, don't give error when we automatically look for .options file
        # with the same name as .proto.
        if options.verbose or had_abspath:
            sys.stderr.write('Options file not found: ' + optfilename + '\n')
        Globals.separate_options = []

    Globals.matched_namemasks = set()

    # Parse the file
    file_options = get_nanopb_suboptions(fdesc, toplevel_options, Names([filename]))
    f = ProtoFile(fdesc, file_options)
    f.optfilename = optfilename

    return f

def process_file(filename, fdesc, options, other_files = {}):
    '''Process a single file.
    filename: The full path to the .proto or .pb source file, as string.
    fdesc: The loaded FileDescriptorSet, or None to read from the input file.
    options: Command line options as they come from OptionsParser.

    Returns a dict:
        {'headername': Name of header file,
         'headerdata': Data for the .h header file,
         'sourcename': Name of the source code file,
         'sourcedata': Data for the .c source code file
        }
    '''
    f = parse_file(filename, fdesc, options)

    # Provide dependencies if available
    for dep in f.fdesc.dependency:
        if dep in other_files:
            f.add_dependency(other_files[dep])

    # Decide the file names
    noext = os.path.splitext(filename)[0]
    headername = noext + options.extension + '.h'
    sourcename = noext + options.extension + '.c'
    headerbasename = os.path.basename(headername)

    # List of .proto files that should not be included in the C header file
    # even if they are mentioned in the source .proto.
    excludes = ['nanopb.proto', 'google/protobuf/descriptor.proto'] + options.exclude
    includes = [d for d in f.fdesc.dependency if d not in excludes]

    headerdata = ''.join(f.generate_header(includes, headerbasename, options))
    sourcedata = ''.join(f.generate_source(headerbasename, options))

    # Check if there were any lines in .options that did not match a member
    unmatched = [n for n,o in Globals.separate_options if n not in Globals.matched_namemasks]
    if unmatched and not options.quiet:
        sys.stderr.write("Following patterns in " + f.optfilename + " did not match any fields: "
                         + ', '.join(unmatched) + "\n")
        if not Globals.verbose_options:
            sys.stderr.write("Use  protoc --nanopb-out=-v:.   to see a list of the field names.\n")

    return {'headername': headername, 'headerdata': headerdata,
            'sourcename': sourcename, 'sourcedata': sourcedata}

def main_cli():
    '''Main function when invoked directly from the command line.'''

    options, filenames = optparser.parse_args()

    if not filenames:
        optparser.print_help()
        sys.exit(1)

    if options.quiet:
        options.verbose = False

    Globals.verbose_options = options.verbose

    for filename in filenames:
        results = process_file(filename, None, options)

        if not options.quiet:
            sys.stderr.write("Writing to " + results['headername'] + " and "
                             + results['sourcename'] + "\n")

        open(results['headername'], 'w').write(results['headerdata'])
        open(results['sourcename'], 'w').write(results['sourcedata'])

def main_plugin():
    '''Main function when invoked as a protoc plugin.'''

    import io, sys
    if sys.platform == "win32":
        import os, msvcrt
        # Set stdin and stdout to binary mode
        msvcrt.setmode(sys.stdin.fileno(), os.O_BINARY)
        msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)

    data = io.open(sys.stdin.fileno(), "rb").read()

    request = plugin_pb2.CodeGeneratorRequest.FromString(data)

    try:
        # Versions of Python prior to 2.7.3 do not support unicode
        # input to shlex.split(). Try to convert to str if possible.
        params = str(request.parameter)
    except UnicodeEncodeError:
        params = request.parameter

    import shlex
    args = shlex.split(params)
    options, dummy = optparser.parse_args(args)

    Globals.verbose_options = options.verbose

    response = plugin_pb2.CodeGeneratorResponse()

    # Google's protoc does not currently indicate the full path of proto files.
    # Instead always add the main file path to the search dirs, that works for
    # the common case.
    import os.path
    options.options_path.append(os.path.dirname(request.file_to_generate[0]))

    # Process any include files first, in order to have them
    # available as dependencies
    other_files = {}
    for fdesc in request.proto_file:
        other_files[fdesc.name] = parse_file(fdesc.name, fdesc, options)

    for filename in request.file_to_generate:
        for fdesc in request.proto_file:
            if fdesc.name == filename:
                results = process_file(filename, fdesc, options, other_files)

                f = response.file.add()
                f.name = results['headername']
                f.content = results['headerdata']

                f = response.file.add()
                f.name = results['sourcename']
                f.content = results['sourcedata']

    io.open(sys.stdout.fileno(), "wb").write(response.SerializeToString())

if __name__ == '__main__':
    # Check if we are running as a plugin under protoc
    if 'protoc-gen-' in sys.argv[0] or '--protoc-plugin' in sys.argv:
        main_plugin()
    else:
        main_cli()




# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""A setup module for the GRPC Python package."""
from distutils import cygwinccompiler
from distutils import extension as _extension
from distutils import util
import os
import os.path
import pkg_resources
import platform
import re
import shlex
import shutil
import sys
import sysconfig

import setuptools
from setuptools.command import egg_info

# Redirect the manifest template from MANIFEST.in to PYTHON-MANIFEST.in.
egg_info.manifest_maker.template = 'PYTHON-MANIFEST.in'

PY3 = sys.version_info.major == 3
PYTHON_STEM = os.path.join('src', 'python', 'grpcio')
CORE_INCLUDE = ('include', '.',)
BORINGSSL_INCLUDE = (os.path.join('third_party', 'boringssl', 'include'),)
ZLIB_INCLUDE = (os.path.join('third_party', 'zlib'),)

# Ensure we're in the proper directory whether or not we're being used by pip.
os.chdir(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, os.path.abspath(PYTHON_STEM))

# Break import-style to ensure we can actually find our in-repo dependencies.
import _spawn_patch
import commands
import grpc_core_dependencies
import grpc_version

_spawn_patch.monkeypatch_spawn()

LICENSE = '3-clause BSD'

# Environment variable to determine whether or not the Cython extension should
# *use* Cython or use the generated C files. Note that this requires the C files
# to have been generated by building first *with* Cython support.
BUILD_WITH_CYTHON = os.environ.get('GRPC_PYTHON_BUILD_WITH_CYTHON', False)

# Environment variable to determine whether or not to enable coverage analysis
# in Cython modules.
ENABLE_CYTHON_TRACING = os.environ.get(
    'GRPC_PYTHON_ENABLE_CYTHON_TRACING', False)

# There are some situations (like on Windows) where CC, CFLAGS, and LDFLAGS are
# entirely ignored/dropped/forgotten by distutils and its Cygwin/MinGW support.
# We use these environment variables to thus get around that without locking
# ourselves in w.r.t. the multitude of operating systems this ought to build on.
# We can also use these variables as a way to inject environment-specific
# compiler/linker flags. We assume GCC-like compilers and/or MinGW as a
# reasonable default.
EXTRA_ENV_COMPILE_ARGS = os.environ.get('GRPC_PYTHON_CFLAGS', None)
EXTRA_ENV_LINK_ARGS = os.environ.get('GRPC_PYTHON_LDFLAGS', None)
if EXTRA_ENV_COMPILE_ARGS is None:
  EXTRA_ENV_COMPILE_ARGS = ''
  if 'win32' in sys.platform and sys.version_info < (3, 5):
    # We use define flags here and don't directly add to DEFINE_MACROS below to
    # ensure that the expert user/builder has a way of turning it off (via the
    # envvars) without adding yet more GRPC-specific envvars.
    # See https://sourceforge.net/p/mingw-w64/bugs/363/
    if '32' in platform.architecture()[0]:
      EXTRA_ENV_COMPILE_ARGS += ' -D_ftime=_ftime32 -D_timeb=__timeb32 -D_ftime_s=_ftime32_s'
    else:
      EXTRA_ENV_COMPILE_ARGS += ' -D_ftime=_ftime64 -D_timeb=__timeb64'
  elif "linux" in sys.platform or "darwin" in sys.platform:
    EXTRA_ENV_COMPILE_ARGS += ' -fno-wrapv'
if EXTRA_ENV_LINK_ARGS is None:
  EXTRA_ENV_LINK_ARGS = ''
  if "linux" in sys.platform or "darwin" in sys.platform:
    EXTRA_ENV_LINK_ARGS += ' -lpthread'
  elif "win32" in sys.platform and sys.version_info < (3, 5):
    msvcr = cygwinccompiler.get_msvcr()[0]
    # TODO(atash) sift through the GCC specs to see if libstdc++ can have any
    # influence on the linkage outcome on MinGW for non-C++ programs.
    EXTRA_ENV_LINK_ARGS += (
        ' -static-libgcc -static-libstdc++ -mcrtdll={msvcr} '
        '-static'.format(msvcr=msvcr))

EXTRA_COMPILE_ARGS = shlex.split(EXTRA_ENV_COMPILE_ARGS)
EXTRA_LINK_ARGS = shlex.split(EXTRA_ENV_LINK_ARGS)

CYTHON_EXTENSION_PACKAGE_NAMES = ()

CYTHON_EXTENSION_MODULE_NAMES = ('grpc._cython.cygrpc',)

CYTHON_HELPER_C_FILES = ()

CORE_C_FILES = tuple(grpc_core_dependencies.CORE_SOURCE_FILES)

EXTENSION_INCLUDE_DIRECTORIES = (
    (PYTHON_STEM,) + CORE_INCLUDE + BORINGSSL_INCLUDE + ZLIB_INCLUDE)

EXTENSION_LIBRARIES = ()
if "linux" in sys.platform:
  EXTENSION_LIBRARIES += ('rt',)
if not "win32" in sys.platform:
  EXTENSION_LIBRARIES += ('m',)
if "win32" in sys.platform:
  EXTENSION_LIBRARIES += ('advapi32', 'ws2_32',)

DEFINE_MACROS = (
    ('OPENSSL_NO_ASM', 1), ('_WIN32_WINNT', 0x600),
    ('GPR_BACKWARDS_COMPATIBILITY_MODE', 1),)
if "win32" in sys.platform:
  DEFINE_MACROS += (('WIN32_LEAN_AND_MEAN', 1),)
  if '64bit' in platform.architecture()[0]:
    DEFINE_MACROS += (('MS_WIN64', 1),)
  elif sys.version_info >= (3, 5):
    # For some reason, this is needed to get access to inet_pton/inet_ntop
    # on msvc, but only for 32 bits
    DEFINE_MACROS += (('NTDDI_VERSION', 0x06000000),)

LDFLAGS = tuple(EXTRA_LINK_ARGS)
CFLAGS = tuple(EXTRA_COMPILE_ARGS)
if "linux" in sys.platform:
  LDFLAGS += ('-Wl,-wrap,memcpy',)
if "linux" in sys.platform or "darwin" in sys.platform:
   CFLAGS += ('-fvisibility=hidden',)
   pymodinit_type = 'PyObject*' if PY3 else 'void'
   pymodinit = '__attribute__((visibility ("default"))) {}'.format(pymodinit_type)
   DEFINE_MACROS += (('PyMODINIT_FUNC', pymodinit),)

# By default, Python3 distutils enforces compatibility of
# c plugins (.so files) with the OSX version Python3 was built with.
# For Python3.4, this is OSX 10.6, but we need Thread Local Support (__thread)
if 'darwin' in sys.platform and PY3:
  mac_target = sysconfig.get_config_var('MACOSX_DEPLOYMENT_TARGET')
  if mac_target and (pkg_resources.parse_version(mac_target) <
                     pkg_resources.parse_version('10.7.0')):
    os.environ['MACOSX_DEPLOYMENT_TARGET'] = '10.7'
    os.environ['_PYTHON_HOST_PLATFORM'] = re.sub(
        r'macosx-[0-9]+\.[0-9]+-(.+)',
        r'macosx-10.7-\1',
        util.get_platform())


def cython_extensions(module_names, extra_sources, include_dirs,
                      libraries, define_macros, build_with_cython=False):
  # Set compiler directives linetrace argument only if we care about tracing;
  # this is due to Cython having different behavior between linetrace being
  # False and linetrace being unset. See issue #5689.
  cython_compiler_directives = {}
  if ENABLE_CYTHON_TRACING:
    define_macros = define_macros + [('CYTHON_TRACE_NOGIL', 1)]
    cython_compiler_directives['linetrace'] = True
  file_extension = 'pyx' if build_with_cython else 'c'
  module_files = [os.path.join(PYTHON_STEM,
                               name.replace('.', '/') + '.' + file_extension)
                  for name in module_names]
  extensions = [
      _extension.Extension(
          name=module_name,
          sources=[module_file] + extra_sources,
          include_dirs=include_dirs, libraries=libraries,
          define_macros=define_macros,
          extra_compile_args=list(CFLAGS),
          extra_link_args=list(LDFLAGS),
      ) for (module_name, module_file) in zip(module_names, module_files)
  ]
  if build_with_cython:
    import Cython.Build
    return Cython.Build.cythonize(
        extensions,
        include_path=include_dirs,
        compiler_directives=cython_compiler_directives)
  else:
    return extensions

CYTHON_EXTENSION_MODULES = cython_extensions(
    list(CYTHON_EXTENSION_MODULE_NAMES),
    list(CYTHON_HELPER_C_FILES) + list(CORE_C_FILES),
    list(EXTENSION_INCLUDE_DIRECTORIES), list(EXTENSION_LIBRARIES),
    list(DEFINE_MACROS), bool(BUILD_WITH_CYTHON))

PACKAGE_DIRECTORIES = {
    '': PYTHON_STEM,
}

INSTALL_REQUIRES = (
    'six>=1.5.2',
    'enum34>=1.0.4',
    # TODO(atash): eventually split the grpcio package into a metapackage
    # depending on protobuf and the runtime component (independent of protobuf)
    'protobuf>=3.0.0',
)

if not PY3:
  INSTALL_REQUIRES += ('futures>=2.2.0',)

SETUP_REQUIRES = INSTALL_REQUIRES + (
    'sphinx>=1.3',
    'sphinx_rtd_theme>=0.1.8',
    'six>=1.10',
)

COMMAND_CLASS = {
    'doc': commands.SphinxDocumentation,
    'build_project_metadata': commands.BuildProjectMetadata,
    'build_py': commands.BuildPy,
    'build_ext': commands.BuildExt,
    'gather': commands.Gather,
}

# Ensure that package data is copied over before any commands have been run:
credentials_dir = os.path.join(PYTHON_STEM, 'grpc', '_cython', '_credentials')
try:
  os.mkdir(credentials_dir)
except OSError:
  pass
shutil.copyfile(os.path.join('etc', 'roots.pem'),
                os.path.join(credentials_dir, 'roots.pem'))

PACKAGE_DATA = {
    # Binaries that may or may not be present in the final installation, but are
    # mentioned here for completeness.
    'grpc._cython': [
        '_credentials/roots.pem',
        '_windows/grpc_c.32.python',
        '_windows/grpc_c.64.python',
    ],
}
PACKAGES = setuptools.find_packages(PYTHON_STEM)

setuptools.setup(
  name='grpcio',
  version=grpc_version.VERSION,
  license=LICENSE,
  ext_modules=CYTHON_EXTENSION_MODULES,
  packages=list(PACKAGES),
  package_dir=PACKAGE_DIRECTORIES,
  package_data=PACKAGE_DATA,
  install_requires=INSTALL_REQUIRES,
  setup_requires=SETUP_REQUIRES,
  cmdclass=COMMAND_CLASS,
)



# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

cimport cpython


cdef class ChannelCredentials:

  def __cinit__(self):
    grpc_init()
    self.c_credentials = NULL
    self.c_ssl_pem_key_cert_pair.private_key = NULL
    self.c_ssl_pem_key_cert_pair.certificate_chain = NULL
    self.references = []

  # The object *can* be invalid in Python if we fail to make the credentials
  # (and the core thus returns NULL credentials). Used primarily for debugging.
  @property
  def is_valid(self):
    return self.c_credentials != NULL

  def __dealloc__(self):
    if self.c_credentials != NULL:
      grpc_channel_credentials_release(self.c_credentials)
    grpc_shutdown()


cdef class CallCredentials:

  def __cinit__(self):
    grpc_init()
    self.c_credentials = NULL
    self.references = []

  # The object *can* be invalid in Python if we fail to make the credentials
  # (and the core thus returns NULL credentials). Used primarily for debugging.
  @property
  def is_valid(self):
    return self.c_credentials != NULL

  def __dealloc__(self):
    if self.c_credentials != NULL:
      grpc_call_credentials_release(self.c_credentials)
    grpc_shutdown()


cdef class ServerCredentials:

  def __cinit__(self):
    grpc_init()
    self.c_credentials = NULL
    self.references = []

  def __dealloc__(self):
    if self.c_credentials != NULL:
      grpc_server_credentials_release(self.c_credentials)
    grpc_shutdown()


cdef class CredentialsMetadataPlugin:

  def __cinit__(self, object plugin_callback, bytes name):
    """
    Args:
      plugin_callback (callable): Callback accepting a service URL (str/bytes)
        and callback object (accepting a Metadata,
        grpc_status_code, and a str/bytes error message). This argument
        when called should be non-blocking and eventually call the callback
        object with the appropriate status code/details and metadata (if
        successful).
      name (bytes): Plugin name.
    """
    grpc_init()
    if not callable(plugin_callback):
      raise ValueError('expected callable plugin_callback')
    self.plugin_callback = plugin_callback
    self.plugin_name = name

  @staticmethod
  cdef grpc_metadata_credentials_plugin make_c_plugin(self):
    cdef grpc_metadata_credentials_plugin result
    result.get_metadata = plugin_get_metadata
    result.destroy = plugin_destroy_c_plugin_state
    result.state = <void *>self
    result.type = self.plugin_name
    cpython.Py_INCREF(self)
    return result

  def __dealloc__(self):
    grpc_shutdown()


cdef class AuthMetadataContext:

  def __cinit__(self):
    grpc_init()
    self.context.service_url = NULL
    self.context.method_name = NULL

  @property
  def service_url(self):
    return self.context.service_url

  @property
  def method_name(self):
    return self.context.method_name

  def __dealloc__(self):
    grpc_shutdown()


cdef void plugin_get_metadata(
    void *state, grpc_auth_metadata_context context,
    grpc_credentials_plugin_metadata_cb cb, void *user_data) with gil:
  def python_callback(
      Metadata metadata, grpc_status_code status,
      bytes error_details):
    cb(user_data, metadata.c_metadata_array.metadata,
       metadata.c_metadata_array.count, status, error_details)
  cdef CredentialsMetadataPlugin self = <CredentialsMetadataPlugin>state
  cdef AuthMetadataContext cy_context = AuthMetadataContext()
  cy_context.context = context
  self.plugin_callback(cy_context, python_callback)

cdef void plugin_destroy_c_plugin_state(void *state) with gil:
  cpython.Py_DECREF(<CredentialsMetadataPlugin>state)

def channel_credentials_google_default():
  cdef ChannelCredentials credentials = ChannelCredentials();
  with nogil:
    credentials.c_credentials = grpc_google_default_credentials_create()
  return credentials

def channel_credentials_ssl(pem_root_certificates,
                            SslPemKeyCertPair ssl_pem_key_cert_pair):
  pem_root_certificates = str_to_bytes(pem_root_certificates)
  cdef ChannelCredentials credentials = ChannelCredentials()
  cdef const char *c_pem_root_certificates = NULL
  if pem_root_certificates is not None:
    c_pem_root_certificates = pem_root_certificates
    credentials.references.append(pem_root_certificates)
  if ssl_pem_key_cert_pair is not None:
    with nogil:
      credentials.c_credentials = grpc_ssl_credentials_create(
          c_pem_root_certificates, &ssl_pem_key_cert_pair.c_pair, NULL)
    credentials.references.append(ssl_pem_key_cert_pair)
  else:
    with nogil:
      credentials.c_credentials = grpc_ssl_credentials_create(
        c_pem_root_certificates, NULL, NULL)
  return credentials

def channel_credentials_composite(
    ChannelCredentials credentials_1 not None,
    CallCredentials credentials_2 not None):
  if not credentials_1.is_valid or not credentials_2.is_valid:
    raise ValueError("passed credentials must both be valid")
  cdef ChannelCredentials credentials = ChannelCredentials()
  with nogil:
    credentials.c_credentials = grpc_composite_channel_credentials_create(
        credentials_1.c_credentials, credentials_2.c_credentials, NULL)
  credentials.references.append(credentials_1)
  credentials.references.append(credentials_2)
  return credentials

def call_credentials_composite(
    CallCredentials credentials_1 not None,
    CallCredentials credentials_2 not None):
  if not credentials_1.is_valid or not credentials_2.is_valid:
    raise ValueError("passed credentials must both be valid")
  cdef CallCredentials credentials = CallCredentials()
  with nogil:
    credentials.c_credentials = grpc_composite_call_credentials_create(
        credentials_1.c_credentials, credentials_2.c_credentials, NULL)
  credentials.references.append(credentials_1)
  credentials.references.append(credentials_2)
  return credentials

def call_credentials_google_compute_engine():
  cdef CallCredentials credentials = CallCredentials()
  with nogil:
    credentials.c_credentials = (
        grpc_google_compute_engine_credentials_create(NULL))
  return credentials

def call_credentials_service_account_jwt_access(
    json_key, Timespec token_lifetime not None):
  json_key = str_to_bytes(json_key)
  cdef CallCredentials credentials = CallCredentials()
  cdef char *json_key_c_string = json_key
  with nogil:
    credentials.c_credentials = (
        grpc_service_account_jwt_access_credentials_create(
            json_key_c_string, token_lifetime.c_time, NULL))
  credentials.references.append(json_key)
  return credentials

def call_credentials_google_refresh_token(json_refresh_token):
  json_refresh_token = str_to_bytes(json_refresh_token)
  cdef CallCredentials credentials = CallCredentials()
  cdef char *json_refresh_token_c_string = json_refresh_token
  with nogil:
    credentials.c_credentials = grpc_google_refresh_token_credentials_create(
        json_refresh_token_c_string, NULL)
  credentials.references.append(json_refresh_token)
  return credentials

def call_credentials_google_iam(authorization_token, authority_selector):
  authorization_token = str_to_bytes(authorization_token)
  authority_selector = str_to_bytes(authority_selector)
  cdef CallCredentials credentials = CallCredentials()
  cdef char *authorization_token_c_string = authorization_token
  cdef char *authority_selector_c_string = authority_selector
  with nogil:
    credentials.c_credentials = grpc_google_iam_credentials_create(
        authorization_token_c_string, authority_selector_c_string, NULL)
  credentials.references.append(authorization_token)
  credentials.references.append(authority_selector)
  return credentials

def call_credentials_metadata_plugin(CredentialsMetadataPlugin plugin):
  cdef CallCredentials credentials = CallCredentials()
  cdef grpc_metadata_credentials_plugin c_plugin = plugin.make_c_plugin()
  with nogil:
    credentials.c_credentials = (
        grpc_metadata_credentials_create_from_plugin(c_plugin, NULL))
  # TODO(atash): the following held reference is *probably* never necessary
  credentials.references.append(plugin)
  return credentials

def server_credentials_ssl(pem_root_certs, pem_key_cert_pairs,
                           bint force_client_auth):
  pem_root_certs = str_to_bytes(pem_root_certs)
  cdef char *c_pem_root_certs = NULL
  if pem_root_certs is not None: 
    c_pem_root_certs = pem_root_certs
  pem_key_cert_pairs = list(pem_key_cert_pairs)
  for pair in pem_key_cert_pairs:
    if not isinstance(pair, SslPemKeyCertPair):
      raise TypeError("expected pem_key_cert_pairs to be sequence of "
                      "SslPemKeyCertPair")
  cdef ServerCredentials credentials = ServerCredentials()
  credentials.references.append(pem_key_cert_pairs)
  credentials.references.append(pem_root_certs)
  credentials.c_ssl_pem_key_cert_pairs_count = len(pem_key_cert_pairs)
  with nogil:
    credentials.c_ssl_pem_key_cert_pairs = (
        <grpc_ssl_pem_key_cert_pair *>gpr_malloc(
            sizeof(grpc_ssl_pem_key_cert_pair) *
                credentials.c_ssl_pem_key_cert_pairs_count
        ))
  for i in range(credentials.c_ssl_pem_key_cert_pairs_count):
    credentials.c_ssl_pem_key_cert_pairs[i] = (
        (<SslPemKeyCertPair>pem_key_cert_pairs[i]).c_pair)
  credentials.c_credentials = grpc_ssl_server_credentials_create(
      c_pem_root_certs, credentials.c_ssl_pem_key_cert_pairs,
      credentials.c_ssl_pem_key_cert_pairs_count,
      GRPC_SSL_REQUEST_AND_REQUIRE_CLIENT_CERTIFICATE_AND_VERIFY if force_client_auth else GRPC_SSL_DONT_REQUEST_CLIENT_CERTIFICATE,
      NULL)
  return credentials




# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Internal utilities for gRPC Python."""

import collections
import threading
import time

import six

import grpc
from grpc import _common
from grpc.framework.foundation import callable_util

_DONE_CALLBACK_EXCEPTION_LOG_MESSAGE = (
    'Exception calling connectivity future "done" callback!')


class RpcMethodHandler(
    collections.namedtuple(
        '_RpcMethodHandler',
        ('request_streaming', 'response_streaming', 'request_deserializer',
         'response_serializer', 'unary_unary', 'unary_stream', 'stream_unary',
         'stream_stream',)),
    grpc.RpcMethodHandler):
  pass


class DictionaryGenericHandler(grpc.GenericRpcHandler):

  def __init__(self, service, method_handlers):
    self._method_handlers = {
        _common.fully_qualified_method(service, method): method_handler
        for method, method_handler in six.iteritems(method_handlers)}

  def service(self, handler_call_details):
    return self._method_handlers.get(handler_call_details.method)


class _ChannelReadyFuture(grpc.Future):

  def __init__(self, channel):
    self._condition = threading.Condition()
    self._channel = channel

    self._matured = False
    self._cancelled = False
    self._done_callbacks = []

  def _block(self, timeout):
    until = None if timeout is None else time.time() + timeout
    with self._condition:
      while True:
        if self._cancelled:
          raise grpc.FutureCancelledError()
        elif self._matured:
          return
        else:
          if until is None:
            self._condition.wait()
          else:
            remaining = until - time.time()
            if remaining < 0:
              raise grpc.FutureTimeoutError()
            else:
              self._condition.wait(timeout=remaining)

  def _update(self, connectivity):
    with self._condition:
      if (not self._cancelled and
          connectivity is grpc.ChannelConnectivity.READY):
        self._matured = True
        self._channel.unsubscribe(self._update)
        self._condition.notify_all()
        done_callbacks = tuple(self._done_callbacks)
        self._done_callbacks = None
      else:
        return

    for done_callback in done_callbacks:
      callable_util.call_logging_exceptions(
          done_callback, _DONE_CALLBACK_EXCEPTION_LOG_MESSAGE, self)

  def cancel(self):
    with self._condition:
      if not self._matured:
        self._cancelled = True
        self._channel.unsubscribe(self._update)
        self._condition.notify_all()
        done_callbacks = tuple(self._done_callbacks)
        self._done_callbacks = None
      else:
        return False

    for done_callback in done_callbacks:
      callable_util.call_logging_exceptions(
          done_callback, _DONE_CALLBACK_EXCEPTION_LOG_MESSAGE, self)

  def cancelled(self):
    with self._condition:
      return self._cancelled

  def running(self):
    with self._condition:
      return not self._cancelled and not self._matured

  def done(self):
    with self._condition:
      return self._cancelled or self._matured

  def result(self, timeout=None):
    self._block(timeout)
    return None

  def exception(self, timeout=None):
    self._block(timeout)
    return None

  def traceback(self, timeout=None):
    self._block(timeout)
    return None

  def add_done_callback(self, fn):
    with self._condition:
      if not self._cancelled and not self._matured:
        self._done_callbacks.append(fn)
        return

    fn(self)

  def start(self):
    with self._condition:
      self._channel.subscribe(self._update, try_to_connect=True)

  def __del__(self):
    with self._condition:
      if not self._cancelled and not self._matured:
        self._channel.unsubscribe(self._update)


def channel_ready_future(channel):
  ready_future = _ChannelReadyFuture(channel)
  ready_future.start()
  return ready_future



#!/usr/bin/env python2.7
# Copyright 2015, Google Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import shutil
import sys
import os
import yaml

sys.dont_write_bytecode = True

boring_ssl_root = os.path.abspath(os.path.join(
    os.path.dirname(sys.argv[0]), 
    '../../third_party/boringssl'))
sys.path.append(os.path.join(boring_ssl_root, 'util'))

try:
  import generate_build_files
except ImportError:
  print yaml.dump({})
  sys.exit()

def map_dir(filename):
  if filename[0:4] == 'src/':
    return 'third_party/boringssl/' + filename[4:]
  else:
    return 'src/boringssl/' + filename

def map_testarg(arg):
  if '/' in arg:
    return 'third_party/boringssl/' + arg
  else:
    return arg

class Grpc(object):

  yaml = None

  def WriteFiles(self, files, asm_outputs):
    self.yaml = {
      '#': 'generated with tools/buildgen/gen_boring_ssl_build_yaml.py',
      'raw_boringssl_build_output_for_debugging': {
        'files': files,
        'asm_outputs': asm_outputs,
      },
      'libs': [
          {
            'name': 'boringssl',
            'build': 'private',
            'language': 'c',
            'secure': 'no',
            'src': sorted(
              map_dir(f)
              for f in files['ssl'] + files['crypto']
            ),
            'headers': sorted(
              map_dir(f)
              for f in files['ssl_headers'] + files['ssl_internal_headers'] + files['crypto_headers'] + files['crypto_internal_headers']
            ),
            'boringssl': True,
            'defaults': 'boringssl',
          },
          {
            'name': 'boringssl_test_util',
            'build': 'private',
            'language': 'c++',
            'secure': 'no',
            'boringssl': True,
            'defaults': 'boringssl',
            'src': [
              map_dir(f)
              for f in sorted(files['test_support'])
            ],
          }
      ] + [
          {
            'name': 'boringssl_%s_lib' % os.path.splitext(os.path.basename(test))[0],
            'build': 'private',
            'secure': 'no',
            'language': 'c' if os.path.splitext(test)[1] == '.c' else 'c++',
            'src': [map_dir(test)],
            'vs_proj_dir': 'test/boringssl',
            'boringssl': True,
            'defaults': 'boringssl',
            'deps': [
                'boringssl_test_util',
                'boringssl',
            ]
          }
          for test in sorted(files['test'])
      ],
      'targets': [
          {
            'name': 'boringssl_%s' % os.path.splitext(os.path.basename(test))[0],
            'build': 'test',
            'run': False,
            'secure': 'no',
            'language': 'c++',
            'src': [],
            'vs_proj_dir': 'test/boringssl',
            'boringssl': True,
            'defaults': 'boringssl',
            'deps': [
                'boringssl_%s_lib' % os.path.splitext(os.path.basename(test))[0],
                'boringssl_test_util',
                'boringssl',
            ]
          }
          for test in sorted(files['test'])
      ],
      'tests': [
          {
            'name': 'boringssl_%s' % os.path.basename(test[0]),
            'args': [map_testarg(arg) for arg in test[1:]],
            'exclude_configs': ['asan'],
            'ci_platforms': ['linux', 'mac', 'posix', 'windows'],
            'platforms': ['linux', 'mac', 'posix', 'windows'],
            'flaky': False,
            'language': 'c++',
            'boringssl': True,
            'defaults': 'boringssl',
            'cpu_cost': 1.0
          }
          for test in files['tests']
      ]
    }


os.chdir(os.path.dirname(sys.argv[0]))
os.mkdir('src')
try:
  for f in os.listdir(boring_ssl_root):
    os.symlink(os.path.join(boring_ssl_root, f),
               os.path.join('src', f))

  g = Grpc()
  generate_build_files.main([g])

  print yaml.dump(g.yaml)

finally:
  shutil.rmtree('src')
